@@ -5,7 +5,7 @@ use warnings;
 use strict;
 use Carp;
 use Config::Simple '-strict';
-use XML::TreeBuilder;
+use XML::TreeBuilder 5.4;
 use I18N::LangTags::List;
 use Term::ANSIColor qw(:constants uncolor);
 use File::Find::Rule;
@@ -18,20 +18,22 @@ use Cwd qw(abs_path);
 use Data::Dumper;
 use File::Copy::Recursive;
 use Encode qw(is_utf8 decode_utf8 encode_utf8);
+use Sort::Versions;
+use version;
 
 use vars
-    qw(@ISA $VERSION @EXPORT @EXPORT_OK $SINGLETON $LOCALISE $SPEC_VERSION);
+    qw(@ISA $VERSION @EXPORT @EXPORT_OK $SINGLETON $LOCALISE $SPEC_VERSION %PARAMS);
 
 $File::Copy::Recursive::KeepMode = 0;
 
-$VERSION = '3.2.1';
+$VERSION = version->declare('v4.2.6');
 @ISA     = qw(Exporter);
 
 @EXPORT
-    = qw(dir_list debug_msg get_all_langs logger help_config maketext new_tree dtd_string rcopy dircopy fcopy rcopy_glob fmove);
+    = qw(dir_list debug_msg get_all_langs logger help_config maketext new_tree dtd_string rcopy dircopy fcopy rcopy_glob fmove dirmove params_as_docbook);
 
 # Track when the SPEC file generation is incompatible.
-$SPEC_VERSION = '3.0';
+$SPEC_VERSION = '4.1';
 
 my $DEFAULT_CONFIG_FILE = 'publican.cfg';
 my $DEBUG               = undef;
@@ -79,7 +81,7 @@ my %PARAM_OLD = (
 
 # All the valid fields in the publican.cfg file
 # constraint: a regex to validate the content
-my %PARAMS = (
+%PARAMS = (
 
     arch => {
         descr => maketext('Arch to filter output on.'),
@@ -120,7 +122,9 @@ my %PARAMS = (
         descr => maketext(
             'The brew dist to use for building the standalone desktop rpm.'),
         default => 'docs-5E',
-
+        alert   => maketext(
+            'This field is deprecated and will be removed from Publican in the future.'
+        ),
     },
     bridgehead_in_toc => {
         descr   => maketext('Display bridge head elements in the TOCs?'),
@@ -147,18 +151,15 @@ my %PARAMS = (
     common_config => {
         descr   => maketext('Path to publican content.'),
         default => Publican::ConfigData->config('datadir'),
-
     },
     common_content => {
         descr   => maketext('Path to publican common content.'),
         default => Publican::ConfigData->config('datadir')
             . '/Common_Content',
-
     },
     condition => {
         descr => maketext(
             'Conditions on which to prune XML before transformation.'),
-
     },
     confidential => {
         descr   => maketext('Is the content confidential?'),
@@ -180,7 +181,7 @@ my %PARAMS = (
     },
     docname => {
         descr => maketext(
-            'Name of this package. Fetched from title tag in xml_lang/TYPE_Info.xml if not set in cfg file.'
+            'Name of this document. Fetched from title tag in xml_lang/TYPE_Info.xml if not set in cfg file.'
         ),
         constraint => '^[0-9a-zA-Z_\-\.\+]+$',
         not_for    => 'brand',
@@ -207,19 +208,20 @@ my %PARAMS = (
         descr => maketext(
             'Version of the DocBook DTD on which this project is based.'),
         default => '4.5',
-
-        #        default => '5.1b4',
-
+    },
+    dt_format => {
+        descr   => maketext('The format to use for the desktop output.'),
+        default => 'html-desktop',
     },
     dt_obsoletes => {
         descr => maketext(
-            'Space-separated list of packages the desktop package obsoletes.'
+            'Space-separated list of packages the desktop RPM obsoletes.'
         ),
 
     },
     dt_requires => {
         descr => maketext(
-            'Space-separated list of packages the desktop package requires.'),
+            'Space-separated list of packages the desktop RPM requires.'),
 
     },
     'ec_id' => {
@@ -285,7 +287,15 @@ my %PARAMS = (
         descr => maketext('os to filter output on.'),
 
     },
-    os_ver  => { descr => maketext('The OS for which to build packages.'), },
+    os_ver => { descr => maketext('The OS for which to build packages.'), },
+    pdf_body_font => {
+        descr   => maketext('The font to use for body text in PDFs.'),
+        default => 'Liberation Sans',
+    },
+    pdf_mono_font => {
+        descr   => maketext('The font to use for mono text in PDFs.'),
+        default => 'Liberation Mono',
+    },
     product => {
         descr => maketext(
             'Product this package covers. Fetched from productname tag in xml_lang/TYPE_Info.xml'
@@ -304,7 +314,12 @@ my %PARAMS = (
     },
     scm => {
         descr => maketext(
-            'Type of repository in which books that form part of a remote set are stored. Supported types: SVN.'
+            'Type of repository in which books that form part of a remote set are stored. Supported types: SVN, GIT.'
+        ),
+    },
+    rev_dir => {
+        descr => maketext(
+            q|By default Revision History is sorted in descending order. Set this to 'asc' or 'ascending' to reverse the sort.|
         ),
     },
     rev_file =>
@@ -397,18 +412,11 @@ my %PARAMS = (
         alert =>
             'web_home is deprecated and will be removed from Publican in the future. Use "web_type: home" instead.',
     },
-    web_style => {
-        descr => maketext(
-            'Splash pages should be generated to be compatible with this web style. Valid values are 1 and 2.'
-        ),
-        constraint => '[1-2]',
-        default    => '1',
-    },
     web_type => {
         descr => maketext(
             'This is a special page for a Publican-generated website, not a standard book. Valid types are home, product, and version.'
         ),
-        constraint => '^(home|product|version|)$',
+        constraint => '^(home|product|version)$',
     },
     web_host => {
         descr => maketext(
@@ -474,24 +482,21 @@ my %PARAMS = (
         descr => maketext(
             'The author name to be shown in drupal book page. It must be a valid drupal username.'
         ),
-        default => 'Redhat',
+        default => 'Publican',
     },
     drupal_menu_title => {
         descr => maketext(
             'Override the bookname that will be shown in the drupal menu.'),
-        default => '',
     },
     drupal_menu_block => {
-        descr => maketext(
-            'The menu where we can find the book. The default value is menu-user-guide'
-        ),
+        descr   => maketext('The menu where we can find the book.'),
         default => 'user-guide',
     },
     drupal_image_path => {
         descr => maketext(
-            'The directory where the image should be stored in drupal server. The default is "sites/default/files/"'
+            'The directory where the image should be stored in drupal server.'
         ),
-        default => 'sites/default/files/',
+        default => '/sites/default/files/documentation/',
     },
 
 );
@@ -609,11 +614,12 @@ sub _load_config {
         if ( defined $tmp ) {
             if ( ref $tmp eq "ARRAY" ) {
                 if ( $#{$tmp} >= 0 ) {
-                    $config->param( $key, join( ',', @{ $Config{$key} } ) );
+                    $config->param( $key,
+                        decode_utf8( join( ',', @{ $Config{$key} } ) ) );
                 }
             }
             elsif ( $tmp ne "" ) {
-                $config->param( $key, $tmp );
+                $config->param( $key, decode_utf8($tmp) );
             }
         }
         elsif ( defined $PARAMS{$key}->{default} ) {
@@ -643,8 +649,31 @@ sub _load_config {
         croak( maketext( "Can't locate required file: [_1]", $info_file ) )
             if ( !-f $info_file );
 
+## BUGBUG DocBook specific stuff should be moved to the DocBook sub classes
+
         my $xml_doc = XML::TreeBuilder->new();
-        $xml_doc->parse_file($info_file);
+        eval { $xml_doc->parse_file($info_file); };
+
+        if ($@) {
+            if ( ref($@) ) {
+
+                # handle a structured error (XML::LibXML::Error object)
+                croak(
+                    maketext(
+                        "FATAL ERROR: [_1]:[_2] in [_3] on line [_4]: [_5]",
+                        $@->domain(),
+                        $@->code(),
+                        $@->file(),
+                        $@->line(),
+                        $@->message(),
+                    )
+                );
+            }
+            else {
+                croak(
+                    maketext( "FATAL ERROR: [_1]: [_2]", $info_file, $@ ) );
+            }
+        }
 
         my $docname = $config->param('docname');
 
@@ -700,7 +729,9 @@ sub _load_config {
         $brand_path =~ s/\"//g;
 
         # Override publican defaults with brand defaults
-        if ( -f "$brand_path/defaults.cfg" ) {
+        if ( -f "$brand_path/defaults.cfg"
+            && ( !-z "$brand_path/defaults.cfg" ) )
+        {
             my $tmp_cfg = new Config::Simple("$brand_path/defaults.cfg")
                 || croak(
                 maketext("Failed to load brand defaults.cfg file") );
@@ -720,7 +751,9 @@ sub _load_config {
         }
 
         # Enforce Brand Overrides
-        if ( -f "$brand_path/overrides.cfg" ) {
+        if ( -f "$brand_path/overrides.cfg"
+            && ( !-z "$brand_path/overrides.cfg" ) )
+        {
             my $tmp_cfg = new Config::Simple("$brand_path/overrides.cfg")
                 || croak(
                 maketext("Failed to load brand overrides.cfg file") );
@@ -831,6 +864,7 @@ sub new {
         $QUIET     = delete( $args->{QUIET} );
         $NOCOLOURS = delete( $args->{NOCOLOURS} );
         my $brand_dir = delete( $args->{brand_dir} );
+        my $allow_network =  delete( $args->{allow_network} );
 
         if ( %{$args} ) {
             croak(
@@ -844,6 +878,8 @@ sub new {
         $self = bless {}, $class;
         $SINGLETON = $self;
 
+        $self->{allow_network} = $allow_network;
+
         # BUGBUG this should be replaced by Publican::Config
         if ( $^O eq 'MSWin32' ) {
             eval { require Win32::TieRegistry; };
@@ -935,7 +971,7 @@ sub debug_msg {
     }
     else {
 
-        #        logger( "$caller: $func, $line\n\t$arg\n", RED );
+##        logger( "$caller: $func, $line\n\t$arg\n", RED );
         logger( "$caller: $arg", RED );
     }
 
@@ -1030,7 +1066,7 @@ sub dir_list {
     my $images = $SINGLETON->param('img_dir');
 
     while ( my $file = $rule->match ) {
-        utf8::decode($file); ## BUGBUG blowing up Archive::Tar.
+        utf8::decode($file);    ## BUGBUG blowing up Archive::Tar.
         push( @filelist, $file )
             unless ( $clean_images
             and $file =~ m{(/$extras/|/icons/|$images/icon.svg)} );
@@ -1046,9 +1082,11 @@ Get all valid language directories.
 =cut
 
 sub get_all_langs {
+    my $no_src_lang = shift;
 
     my ( $handle, %filelist, @langs );
-    my $tmp_dir = $SINGLETON->param('tmp_dir');
+    my $tmp_dir  = $SINGLETON->param('tmp_dir');
+    my $xml_lang = $SINGLETON->param('xml_lang');
 
     opendir( $handle, '.' )
         || croak( maketext( "Can't open directory: [_1]", $@ ) );
@@ -1059,11 +1097,12 @@ sub get_all_langs {
         if ( -d $dir ) {
             next
                 if ( $dir
-                =~ /^(\.|\.\.|pot|$tmp_dir|xsl|\..*|CVS|publish|book_templates|trans_drop)$/
+                =~ /^(\.|\.\.|pot|$tmp_dir|xsl|\..*|CVS|publish|book_templates|trans_drop|rpm_templates)$/
                 );
 
             if ( valid_lang($dir) ) {
-                push( @langs, $dir );
+                push( @langs, $dir )
+                    unless ( $no_src_lang && ( $dir eq $xml_lang ) );
             }
             else {
                 logger( maketext( "Skipping unknown language: [_1]", $dir )
@@ -1122,7 +1161,7 @@ sub maketext {
     my @params = @_;
 
     if ($LOCALISE) {
-        return ( $LOCALISE->maketext( $string, @params ) );
+        return ( decode_utf8( $LOCALISE->maketext( $string, @params ) ) );
     }
     else {
         carp( RED, "Warning localisation not enabled!\n", RESET );
@@ -1131,369 +1170,6 @@ sub maketext {
     return ($string);
 }
 
-=head2 get_abstract
-
-Return the abstract for the supplied language with all white space truncated.
-
-=cut
-
-sub get_abstract {
-    my ( $self, $args ) = @_;
-
-    my $lang = delete( $args->{lang} )
-        || croak( maketext("lang is a mandatory argument") );
-
-    if ( %{$args} ) {
-        croak(
-            maketext(
-                "unknown arguments: [_1]", join( ", ", keys %{$args} )
-            )
-        );
-    }
-
-    my $tmp_dir = $self->param('tmp_dir');
-    my $info_file
-        = "$tmp_dir/$lang/xml/" . $self->param('type') . '_Info.xml';
-    $info_file = "$tmp_dir/$lang/xml/" . $self->param('info_file')
-        if ( $self->param('info_file') );
-
-    croak( maketext("abstract can not be calculated before building.") )
-        unless ( -f $info_file );
-
-    my $xsl_file = $self->param('common_config') . "/xsl/abstract.xsl";
-
-    my $abstract = $self->run_xslt(
-        { xml_file => $info_file, xsl_file => $xsl_file } );
-
-    # tidy up white space
-    $abstract =~ s/^[ \t]*//gm;
-    $abstract =~ s/^\n\n+//;
-    $abstract =~ s/\n\n+$//;
-    $abstract =~ s/\n\n\n/\n\n/g;
-    $abstract =~ s/[ \t][ \t]+/ /gm;
-
-    # RPM doesn't like non-breaking-space
-    $abstract =~ s/\x{A0}/ /gm;
-    return ($abstract);
-}
-
-=head2 get_subtitle
-
-Return the subtitle for the supplied language with white space truncated.
-
-=cut
-
-sub get_subtitle {
-    my ( $self, $args ) = @_;
-
-    my $lang = delete( $args->{lang} )
-        || croak( maketext("lang is a mandatory argument") );
-
-    if ( %{$args} ) {
-        croak(
-            maketext(
-                "unknown arguments: [_1]", join( ", ", keys %{$args} )
-            )
-        );
-    }
-
-    my $tmp_dir = $self->param('tmp_dir');
-    my $info_file
-        = "$tmp_dir/$lang/xml/" . $self->param('type') . '_Info.xml';
-    $info_file = "$tmp_dir/$lang/xml/" . $self->param('info_file')
-        if ( $self->param('info_file') );
-
-    croak( maketext("subtitle can not be calculated before building.") )
-        unless ( -f $info_file );
-
-    my $xsl_file = $self->param('common_config') . "/xsl/subtitle.xsl";
-
-    my $subtitle = $self->run_xslt(
-        { xml_file => $info_file, xsl_file => $xsl_file } );
-
-    # tidy up white space
-    $subtitle =~ s/^\s*//gm;
-
-    # RPM doesn't like non-breaking-space
-    $subtitle =~ s/\x{A0}/ /gm;
-    $subtitle =~ s/\s+/ /;
-
-    return ($subtitle);
-}
-
-=head2 get_author_list
-
-Return the author list for the supplied language.
-
-=cut
-
-sub get_author_list {
-    my ( $self, $args ) = @_;
-
-    my $lang = delete( $args->{lang} )
-        || croak( maketext("lang is a mandatory argument") );
-
-    if ( %{$args} ) {
-        croak(
-            maketext(
-                "unknown arguments: [_1]", join( ", ", keys %{$args} )
-            )
-        );
-    }
-
-    my @authors;
-
-    my $tmp_dir = $self->param('tmp_dir');
-    my $file    = "$tmp_dir/$lang/xml/Author_Group.xml";
-
-    croak( maketext("ERROR: Cannot find Author file Author_Group.xml.") )
-        unless ( -f $file );
-
-    my $xml_doc = XML::TreeBuilder->new(
-        { 'NoExpand' => "0", 'ErrorContext' => "2" } );
-    $xml_doc->parse_file($file);
-
-    foreach my $author (
-        $xml_doc->root()->look_down( "_tag", qr/author|corpauthor/ ) )
-    {
-        if ( $author->tag() eq 'corpauthor' ) {
-            my $name;
-
-            eval { $name = $author->as_text; };
-            if ($@) {
-                croak(
-                    maketext(
-                        "corpauthor can not be converted to text as expected."
-                    )
-                );
-            }
-            push( @authors, "$name" );
-        }
-        else {
-            my ( $fn, $sn );
-            eval { $fn = $author->look_down( "_tag", 'firstname' )->as_text; };
-            if ($@) {
-                croak(
-                    maketext(
-                        "Author’s firstname not found in Author_Group.xml as expected."
-                    )
-                );
-            }
-
-            eval { $sn = $author->look_down( "_tag", 'surname' )->as_text; };
-            if ($@) {
-                croak(
-                    maketext(
-                        "Author’s surname not found in Author_Group.xml as expected."
-                    )
-                );
-            }
-
-            push( @authors, "$fn $sn" );
-        }
-    }
-
-    unless (@authors) {
-        croak( maketext("Did not find any authors in Author_Group.xml") );
-    }
-
-    return (@authors);
-}
-
-=head2 get_contributors
-
-Return the contributor hash for the supplied language.
-
-=cut
-
-sub get_contributors {
-    my ( $self, $args ) = @_;
-
-    my $lang = delete( $args->{lang} )
-        || croak( maketext("lang is a mandatory argument") );
-
-    if ( %{$args} ) {
-        croak(
-            maketext(
-                "unknown arguments: [_1]", join( ", ", keys %{$args} )
-            )
-        );
-    }
-
-    my %contributors;
-
-    my $tmp_dir = $self->param('tmp_dir');
-    my $file    = "$tmp_dir/$lang/xml/Author_Group.xml";
-
-    croak(
-        maketext("contributor list can not be calculated before building.") )
-        unless ( -f $file );
-
-    my $xml_doc = XML::TreeBuilder->new(
-        { 'NoExpand' => "0", 'ErrorContext' => "2" } );
-    $xml_doc->parse_file($file);
-
-    foreach my $node ( $xml_doc->root()
-        ->look_down( "_tag", qr/^(?:author|editor|othercredit|corpauthor)$/ )
-        )
-    {
-        my %person;
-        if ( $node->attr('class') ) {
-            my $role = $node->attr('class');
-            if ( $role eq "copyeditor" ) {
-                $person{role} = maketext("Copy Editor");
-            }
-            elsif ( $role eq "graphicdesigner" ) {
-                $person{role} = maketext("Graphic Designer");
-            }
-            elsif ( $role eq "productioneditor" ) {
-                $person{role} = maketext("Production Editor");
-            }
-            elsif ( $role eq "technicaleditor" ) {
-                $person{role} = maketext("Technical Editor");
-            }
-            elsif ( $role eq "translator" ) {
-                $person{role} = maketext("Translator");
-            }
-        }
-
-        my @fields
-            = qw/firstname surname email contrib orgname orgdiv corpauthor/;
-        foreach my $field (@fields) {
-            my $field_node = $node->look_down( "_tag", $field );
-            if ($field_node) {
-                $person{$field} = $field_node->as_text();
-            }
-        }
-
-        push( @{ $contributors{ $node->tag() } }, \%person );
-    }
-
-    return ( \%contributors );
-}
-
-=head2 get_keywords
-
-Return the contributor hash for the supplied language.
-
-=cut
-
-sub get_keywords {
-    my ( $self, $args ) = @_;
-
-    my $lang = delete( $args->{lang} )
-        || croak( maketext("lang is a mandatory argument") );
-
-    if ( %{$args} ) {
-        croak(
-            maketext(
-                "unknown arguments: [_1]", join( ", ", keys %{$args} )
-            )
-        );
-    }
-
-    my @keywords;
-
-    my $tmp_dir = $self->param('tmp_dir');
-    my $info
-        = ( $self->param('info_file') || $self->param('type') . '_Info.xml' );
-    my $file = "$tmp_dir/$lang/xml/$info";
-
-    croak( maketext("keyword list can not be calculated before building.") )
-        unless ( -f $file );
-
-    my $xml_doc = XML::TreeBuilder->new(
-        { 'NoExpand' => "0", 'ErrorContext' => "2" } );
-    $xml_doc->parse_file($file);
-
-    foreach my $node ( $xml_doc->root()->look_down( "_tag", 'keyword' ) ) {
-        push( @keywords, $node->as_text() );
-    }
-
-    return (@keywords);
-}
-
-=head2 get_legalnotice
-
-Return the legal notice for the supplied language.
-
-=cut
-
-sub get_legalnotice {
-    my ( $self, $args ) = @_;
-
-    my $lang = delete( $args->{lang} )
-        || croak( maketext("lang is a mandatory argument") );
-
-    if ( %{$args} ) {
-        croak(
-            maketext(
-                "unknown arguments: [_1]", join( ", ", keys %{$args} )
-            )
-        );
-    }
-
-    my @keywords;
-
-    my $tmp_dir = $self->param('tmp_dir');
-    my $file    = "$tmp_dir/$lang/xml/Common_Content/Legal_Notice.xml";
-
-    croak( maketext("Legal notice can not be calculated before building.") )
-        unless ( -f $file );
-
-    my $xml_doc = XML::TreeBuilder->new(
-        { 'NoExpand' => "0", 'ErrorContext' => "2" } );
-    $xml_doc->parse_file($file);
-
-## BUGBUG should this be using run_xslt to get the formatted legal notice?
-
-    return (
-        $xml_doc->root()->look_down( "_tag", 'legalnotice' )->as_text() );
-}
-
-=head2 get_draft
-
-Is the book in draft mode?.
-
-=cut
-
-sub get_draft {
-    my ( $self, $args ) = @_;
-
-    my $lang = delete( $args->{lang} )
-        || croak( maketext("lang is a mandatory argument") );
-
-    if ( %{$args} ) {
-        croak(
-            maketext(
-                "unknown arguments: [_1]", join( ", ", keys %{$args} )
-            )
-        );
-    }
-
-    my $main_file = $self->param('mainfile');
-    my $draft     = 0;
-
-    my $tmp_dir = $self->param('tmp_dir');
-    my $file    = "$tmp_dir/$lang/xml/$main_file.xml";
-
-    croak(
-        maketext(
-            "Main XML file ([_1]) can not be calculated before building.",
-            "$main_file.xml"
-        )
-    ) unless ( -f $file );
-
-    my $xml_doc = XML::TreeBuilder->new(
-        { 'NoExpand' => "0", 'ErrorContext' => "2" } );
-    $xml_doc->parse_file($file);
-
-## BUGBUG should this be using run_xslt to get the formatted legal notice?
-    $draft = ( $xml_doc->root()->attr('status')
-            && $xml_doc->root()->attr('status') eq 'draft' );
-    return ($draft);
-}
-
 =head2 run_xslt
 
 Apply the supplied xslt file to the supplied XML and return a string of the output.
@@ -1515,9 +1191,10 @@ sub run_xslt {
         );
     }
 
-    my $parser = XML::LibXML->new();
+    my $parser = XML::LibXML->new(no_network => !$self->{allow_network});
     my $xslt   = XML::LibXSLT->new();
     $parser->expand_entities(1);
+    
     my $source;
     eval { $source = $parser->parse_file($xml_file); };
 
@@ -1537,11 +1214,32 @@ sub run_xslt {
             );
         }
         else {
-            croak( maketext( "FATAL ERROR: [_1]", $@ ) );
+            croak( maketext( "FATAL ERROR: [_1]: [_2]", $xml_file, $@ ) );
         }
     }
 
-    my $style_doc = $parser->parse_file($xsl_file);
+    my $style_doc;
+    eval { $style_doc = $parser->parse_file($xsl_file); };
+
+    if ($@) {
+        if ( ref($@) ) {
+
+            # handle a structured error (XML::LibXML::Error object)
+            croak(
+                maketext(
+                    "FATAL ERROR: [_1]:[_2] in [_3] on line [_4]: [_5]",
+                    $@->domain(),
+                    $@->code(),
+                    $@->file(),
+                    $@->line(),
+                    $@->message(),
+                )
+            );
+        }
+        else {
+            croak( maketext( "FATAL ERROR: [_1]: [_2]", $xsl_file, $@ ) );
+        }
+    }
 
     if ( $^O eq 'MSWin32' ) {
         eval { require Win32::TieRegistry; };
@@ -1556,7 +1254,7 @@ sub run_xslt {
         my $key = new Win32::TieRegistry( "LMachine\\Software\\Publican",
             { Delimiter => "\\" } );
 
-        my $new_href = 'file:///D:/Data/temp/Redhat/docbook-xsl-1.75.2';
+        my $new_href = 'file:///D:/Data/temp/Publican/docbook-xsl-1.75.2';
         if ( $key and $key->GetValue("xsl_path") ) {
             $new_href = 'file:///' . $key->GetValue("xsl_path");
             $new_href =~ s/ /%20/g;
@@ -1591,9 +1289,13 @@ TODO: Make XmlClean use this.
 sub new_tree {
 
     my $store_comments = ( shift() || 0 );
+    my $noexpand = shift();
 
     my $xml_doc = XML::TreeBuilder->new(
-        { 'NoExpand' => "1", 'ErrorContext' => "2" } );
+        { 'NoExpand' => !$noexpand, 'ErrorContext' => "2" } );
+    $xml_doc->store_pis(1);
+    $xml_doc->store_cdata(1);
+
     my $empty_element_map = $xml_doc->_empty_element_map;
     $empty_element_map->{'xref'}       = 1;
     $empty_element_map->{'index'}      = 1;
@@ -1616,6 +1318,8 @@ Parameters:
 	dtdver	 The DTD version
 	ent_file An entity file to include (optional)
 
+## BUGBUG this should be moved to the DocBook sub classes
+
 =cut
 
 sub dtd_string {
@@ -1638,11 +1342,6 @@ sub dtd_string {
     my $uri = qq|http://www.oasis-open.org/docbook/xml/$dtdver/docbookx.dtd|;
     my $dtd_type = qq|-//OASIS//DTD DocBook XML V$dtdver//EN|;
 
-    if ( $dtdver =~ m/^5/ ) {
-        $dtd_type = qq|-//OASIS//DTD DocBook XML $dtdver//EN|;
-        $uri      = qq|http://docbook.org/xml/$dtdver/rng/docbook.rng|;
-    }
-
     # TODO Maynot be necessary
     if ( $^O eq 'MSWin32' ) {
         eval { require Win32::TieRegistry; };
@@ -1675,24 +1374,47 @@ sub dtd_string {
 <!DOCTYPE $tag PUBLIC "$dtd_type" "$uri" [
 DTDHEAD
 
+    if ( $dtdver =~ m/^5/ ) {
+        # make sure docbook4 entities still work
+        $dtd = <<DTDHEAD;
+<?xml version='1.0' encoding='utf-8' ?>
+<!DOCTYPE $tag [
+DTDHEAD
+    }
+
+    # BUGBUG if we put the DB ENT first it breaks the builds ...
+    my $before = 0;
+
+    if ( $before == 1 &&  $dtdver =~ m/^5/ ) {
+        # make sure docbook4 entities still work
+        $dtd .= <<DTDHEAD;
+<!ENTITY % sgml.features "IGNORE">
+<!ENTITY % xml.features "INCLUDE">
+<!ENTITY % DOCBOOK_ENTS PUBLIC "-//OASIS//ENTITIES DocBook Character Entities V4.5//EN" "http://www.oasis-open.org/docbook/xml/4.5/dbcentx.mod">
+%DOCBOOK_ENTS;
+DTDHEAD
+    }
+
+
     # handle entity file
     if ($ent_file) {
-        if ($cleaning) {
-            $dtd .= <<ENT;
+        $dtd .= <<ENT;
 <!ENTITY % BOOK_ENTITIES SYSTEM "$ent_file">
 %BOOK_ENTITIES;
 ENT
-        }
-        else {
-            my $INFILE;
-            open( $INFILE, "<:encoding(UTF-8)", "$ent_file" )
-                || croak(
-                maketext( "Could not open [_1] for input!", $ent_file ) );
-            my @lines = <$INFILE>;
-            $INFILE->close();
-            $dtd .= join( "", @lines );
-        }
     }
+
+    if ( $before == 0 &&  $dtdver =~ m/^5/ ) {
+        # make sure docbook4 entities still work
+        $dtd .= <<DTDHEAD;
+<!ENTITY % sgml.features "IGNORE">
+<!ENTITY % xml.features "INCLUDE">
+<!ENTITY % DOCBOOK_ENTS PUBLIC "-//OASIS//ENTITIES DocBook Character Entities V4.5//EN" "http://www.oasis-open.org/docbook/xml/4.5/dbcentx.mod">
+%DOCBOOK_ENTS;
+DTDHEAD
+    }
+
+
     $dtd .= <<DTDTAIL;
 ]>
 DTDTAIL
@@ -1735,6 +1457,8 @@ sub print_banned_tags {
 
 Add a full entry in to the revision history.
 
+## BUGBUG this should be moved to the DocBook sub classes
+
 =cut
 
 sub add_revision {
@@ -1787,26 +1511,19 @@ sub add_revision {
                 [ 'surname',   $surname ],
                 [ 'email',     $email ],
             ],
-            [   'revdescription',
-                [   'simplelist',
-                    map {
-                        [   'member',
-                            eval {
-                                XML::TreeBuilder->new(
-                                    {   'NoExpand'     => "1",
-                                        'ErrorContext' => "2"
-                                    }
-                                )->parse("<rubbish>$_</rubbish>");
-                            }
-                        ]
-                        } @{$members},
-                ],
-            ],
+            [ 'revdescription', ['simplelist'], ],
         ],
     );
 
-    foreach my $node ( $revision->root()->look_down( "_tag", 'rubbish' ) ) {
-        $node->replace_with_content()->delete();
+    my $list = $revision->root()->look_down( "_tag", 'simplelist' );
+    foreach my $member ( @{$members} ) {
+        my $mem = XML::TreeBuilder->new(
+            {   'NoExpand'     => "1",
+                'ErrorContext' => "2"
+            }
+        );
+        $mem->parse("<member>$member</member>");
+        $list->push_content($mem);
     }
 
     my $dtdver    = $self->param('dtdver');
@@ -1824,14 +1541,33 @@ sub add_revision {
     my $rev_doc = new_tree();
 
     if ( -f $rev_file ) {
-        $rev_doc->parse_file($rev_file);
+        eval { $rev_doc->parse_file($rev_file); };
+
+        if ($@) {
+            if ( ref($@) ) {
+
+                # handle a structured error (XML::LibXML::Error object)
+                croak(
+                    maketext(
+                        "FATAL ERROR: [_1]:[_2] in [_3] on line [_4]: [_5]",
+                        $@->domain(),
+                        $@->code(),
+                        $@->file(),
+                        $@->line(),
+                        $@->message(),
+                    )
+                );
+            }
+            else {
+                croak( maketext( "FATAL ERROR: [_1]: [_2]", $rev_file, $@ ) );
+            }
+        }
     }
     else {
         $rev_doc->root()->tag('appendix');
-        my $rev_hist
-            = XML::Element->new_from_lol(
-            [ 'title', $locale->maketext('Revision History') ],
-            );
+        my $rev_hist = XML::Element->new_from_lol(
+            [ 'title', decode_utf8( $locale->maketext('Revision History') ) ],
+        );
 
         $rev_doc->root()->push_content($rev_hist);
 
@@ -1846,7 +1582,12 @@ sub add_revision {
         croak( maketext( "revhistory not found in [_1]", $rev_file ) );
     }
 
-    $node->unshift_content($revision);
+    if ( $self->param('rev_dir') && $self->param('rev_dir') =~ /^asc/i ) {
+        $node->push_content($revision);
+    }
+    else {
+        $node->unshift_content($revision);
+    }
 
     my $OUTDOC;
     open( $OUTDOC, ">:encoding(UTF-8)", "$rev_file" )
@@ -1877,6 +1618,8 @@ Parameters: language, bump.
 
 If bump is set the returned revision will increment before it's returned.
 
+## BUGBUG this should be moved to the DocBook sub classes
+
 =cut
 
 sub get_ed_rev {
@@ -1895,21 +1638,45 @@ sub get_ed_rev {
 
     my $rev_doc = XML::TreeBuilder->new();
     eval { $rev_doc->parse_file($rev_file); };
+
     if ($@) {
-        croak( maketext( "FATAL ERROR: [_1]: [_2]", $rev_file, $@ ) );
+        if ( ref($@) ) {
+
+            # handle a structured error (XML::LibXML::Error object)
+            croak(
+                maketext(
+                    "FATAL ERROR: [_1]:[_2] in [_3] on line [_4]: [_5]",
+                    $@->domain(),
+                    $@->code(),
+                    $@->file(),
+                    $@->line(),
+                    $@->message(),
+                )
+            );
+        }
+        else {
+            croak( maketext( "FATAL ERROR: [_1]: [_2]", $rev_file, $@ ) );
+        }
     }
 
-    my $VR = eval {
-        $rev_doc->root()->look_down( "_tag", "revnumber" )->as_text();
-    };
-    if ($@) {
-        croak( maketext( "revnumber not found in [_1]", $rev_file ) );
-    }
+    my @revs = map { $_->as_text() }
+        sort { versioncmp( $b->as_text(), $a->as_text() ) }
+        ( $rev_doc->root()->look_down( "_tag", "revnumber" ) );
+
+    my $VR = shift(@revs);
+
+    croak(
+        maketext(
+            "FATAL ERROR: revnumber missing or empty, it must match the required format of '[_1]'",
+            '^([0-9.]*)-([0-9.]*)$/'
+        )
+    ) if ( !$VR || $VR eq '' );
 
     $VR =~ /^([0-9.]*)-([0-9.]*)$/ || croak(
         maketext(
-            "revnumber ([_1]) does not match the required format of '[_2]'",
-            $VR, '^([0-9.]*)-([0-9.]*)$/'
+            "FATAL ERROR: revnumber ([_1]) does not match the required format of '[_2]'",
+            $VR,
+            '^([0-9.]*)-([0-9.]*)$/'
         )
     );
 
@@ -2022,6 +1789,108 @@ sub dircopy {
     return;
 }
 
+=head2 dirmove
+
+UTF8 escape calls to File::Copy::Recursive
+
+=cut
+
+sub dirmove {
+    my ( $from, $to ) = @_;
+
+    File::Copy::Recursive::dirmove( encode_utf8($from), encode_utf8($to) )
+        || croak(
+        maketext(
+            "Can not move directory [_1] to [_2] due to error: [_3]",
+            $from, $to, $@
+        )
+        );
+
+    return;
+}
+
+=head2 params_as_docbook
+
+Returns DocBook chunks describing all the configuration options. Used to generate autodocs.
+
+=cut
+
+sub params_as_docbook {
+    my ( $gen_list, $brand_list, $web_list ) = @_;
+
+    foreach my $key ( sort( keys(%PARAMS) ) ) {
+        my $entry = XML::Element->new_from_lol(
+            [ 'varlistentry', {id => $key},[ 'term', "$key" ] ] );
+
+        if ( defined( $PARAMS{$key}->{limit_to} )
+            && $PARAMS{$key}->{limit_to} eq 'brand' )
+        {
+            $brand_list->push_content($entry);
+        }
+        elsif ( defined( $PARAMS{$key}->{limit_to} )
+            && $PARAMS{$key}->{limit_to} eq 'site' )
+        {
+            $web_list->push_content($entry);
+        }
+        else {
+            $gen_list->push_content($entry);
+        }
+        my $item = XML::Element->new_from_lol(
+            [ 'listitem', [ 'para', $PARAMS{$key}->{descr} ] ] );
+
+        $entry->push_content($item);
+
+        if ( defined( $PARAMS{$key}->{default} ) ) {
+            my $def = XML::Element->new_from_lol(
+                [   'para',
+                    maketext(
+                        "The default value for this parameter is: [_1]",
+                        $PARAMS{$key}->{default}
+                    )
+                ]
+            );
+
+            $item->push_content($def);
+        }
+
+        if ( defined( $PARAMS{$key}->{constraint} ) ) {
+            my $constraint = XML::Element->new_from_lol(
+                [   'para',
+                    maketext(
+                        "This parameter is constrained with the following regular expression: [_1]",
+                        $PARAMS{$key}->{constraint}
+                    )
+                ]
+            );
+
+            $item->push_content($constraint);
+        }
+
+        if ( defined( $PARAMS{$key}->{not_for} ) ) {
+            my $info = XML::Element->new_from_lol(
+                [   'tip',
+                    [   'para',
+                        maketext(
+                            "This field is not supported for: [_1].",
+                            $PARAMS{$key}->{not_for}
+                        )
+                    ]
+                ]
+            );
+
+            $item->push_content($info);
+        }
+
+        if ( defined( $PARAMS{$key}->{alert} ) ) {
+            my $warn = XML::Element->new_from_lol(
+                [ 'warning', [ 'para', $PARAMS{$key}->{alert} ] ] );
+
+            $item->push_content($warn);
+        }
+    }
+
+}
+
 1;    # Magic true value required at end of module
 __END__
 
