@@ -11,42 +11,32 @@ use Publican::Localise;
 use File::Path;
 use Term::ANSIColor qw(:constants);
 use DateTime;
-use Locale::PO;
+use Locale::PO 0.24;
 use XML::TreeBuilder;
 use String::Similarity;
 use Encode qw(is_utf8 decode_utf8 encode_utf8);
 
-use vars qw($VERSION);
-
-$VERSION = '0.2';
-
 # What tags do we translate?
 my $TRANSTAGS
-    = qr/^(?:ackno|bridgehead|caption|conftitle|contrib|entry|firstname|glossterm|indexterm|jobtitle|keyword|label|lastname|lineannotation|lotentry|member|orgdiv|orgname|othername|para|phrase|productname|refclass|refdescriptor|refentrytitle|refmiscinfo|refname|refpurpose|releaseinfo|revremark|screeninfo|secondaryie|seealsoie|seeie|seg|segtitle|simpara|subtitle|surname|term|termdef|tertiaryie|textobject|title|titleabbrev|screen|programlisting|literallayout)$/;
+    = qr/^(?:ackno|bridgehead|caption|conftitle|contrib|entry|firstname|glossentry|indexterm|jobtitle|keyword|label|lastname|lineannotation|lotentry|member|orgdiv|orgname|othername|para|phrase|productname|refclass|refdescriptor|refentrytitle|refmiscinfo|refname|refpurpose|releaseinfo|revremark|screeninfo|secondaryie|seealsoie|seeie|seg|segtitle|simpara|subtitle|surname|td|term|termdef|tertiaryie|textobject|th|title|titleabbrev|screen|programlisting|literallayout|simplelist)$/;
 
 # Blocks that contain translatable tags that need to be kept inline
 my $IGNOREBLOCKS
     = qr/^(?:footnote|citerefentry|indexterm|orgname|productname|phrase|textobject)$/;
 
 # Preserve white space in these tags
-my $VERBATIM = qr/^(?:screen|programlisting|literallayout)$/;
+my $VERBATIM = qr/^(?:screen|programlisting|literallayout|simplelist)$/;
 
 =head1 NAME
 
 Publican::Translate - Module for manipulating POT and PO files.
 
-
-=head1 VERSION
-
-This document describes Publican::Translate version $VERSION
-
-
 =head1 SYNOPSIS
 
 	use Publican::Translate;
 	my $po = Publican::Translate->new();
 
- 	$po->update_pot();
+	$po->update_pot();
 	$po->update_po({ langs => 'fr-FR,de-DE' });
 	$po->update_po({ langs => 'all' });
 	$po->merge_xml({ lang  => 'fr-FR' });
@@ -67,11 +57,23 @@ Create a new Publican::Translate object.
 
 sub new {
     my ( $this, $args ) = @_;
+
+    my $showfuzzy = delete( $args->{showfuzzy} );
+
+    if ( %{$args} ) {
+        croak(
+            maketext(
+                "unknown arguments: [_1]", join( ", ", keys %{$args} )
+            )
+        );
+    }
+
     my $class = ref($this) || $this;
     my $self = bless {}, $class;
 
     my $publican = Publican->new();
-    $self->{publican} = $publican;
+    $self->{publican}  = $publican;
+    $self->{showfuzzy} = $showfuzzy;
 
     return $self;
 }
@@ -128,17 +130,20 @@ sub update_pot {
         my $pot_file = $xml_file;
         $pot_file =~ s/\.xml/\.pot/;
         $pot_file =~ s/^$source_dir/pot/;
-        $pot_file =~ m|^(.*)/[^/]+$|;
-        my $path = $1;
+        $pot_file =~ m|^(.*)/([^/]+)$|;
+        my $path     = $1;
+        my $filename = $2;
         mkpath($path) if ( !-d $path );
 
         my $xml_doc = Publican::Builder::new_tree();
+        $xml_doc->store_cdata(1);
         $xml_doc->parse_file($xml_file)
             || croak(
             maketext( "Can't open file [_1]. Error: [_2]", $xml_file, $@ ) );
         $xml_doc->pos( $xml_doc->root() );
 
-        my $msg_list = $self->get_msgs( { doc => $xml_doc } );
+        my $msg_list
+            = $self->get_msgs( { doc => $xml_doc, filename => $filename } );
 ##debug_msg( "hash: " . $msg_list->content_list() . "\n\n" );
         $self->print_msgs( { msg_list => $msg_list, pot_file => $pot_file } );
 
@@ -172,6 +177,7 @@ sub po2xml {
         || croak( maketext("po_file is a mandatory argument") );
     my $out_file = delete( $args->{out_file} )
         || croak( maketext("out_file is a mandatory argument") );
+    my $ent_file = delete( $args->{ent_file} );
 
     if ( %{$args} ) {
         croak(
@@ -188,6 +194,14 @@ sub po2xml {
             . "\n"
     );
 
+    my $path = undef;
+    if ( $out_file =~ m|^(.*/xml)/(.*\/)[^\/]*\.xml| ) {
+        $path = $2;
+        $path =~ s|[^/]*/|\.\./|g;
+    }
+
+    $ent_file = "$path$ent_file" if ($path);
+
     my $dtdver = $self->{publican}->param('dtdver');
 
     my $out_doc = Publican::Builder::new_tree();
@@ -196,25 +210,24 @@ sub po2xml {
         maketext( "Can't open file [_1]. Error: [_2]", $xml_file, $@ ) );
     $out_doc->pos( $out_doc->root() );
 
-    my $msgids = Locale::PO->load_file_ashash($po_file);
+    my $msgids = Locale::PO->load_file_ashash( $po_file, 'UTF-8' );
     foreach my $key ( keys( %{$msgids} ) ) {
-##debug_msg("key: $key\n");
-##debug_msg("is utf8  key " . utf8::is_utf8($key) . "\n");
         my $msgref = $msgids->{$key};
-##debug_msg("is utf8 msgref " . utf8::is_utf8($msgref->msgstr()) . "\n");
         if ( $msgref->obsolete() ) {
-##            debug_msg("Deleting obsolete msg_id: $key\n");
             delete( $msgids->{$key} );
         }
-        if ( $msgref->fuzzy() ) {
-##            debug_msg("no fuzzy matches: $key\n");
+        if ( $msgref->fuzzy() && !$self->{showfuzzy} ) {
             $msgref->msgstr("");
         }
     }
 
-##debug_msg( "hash: " . join( "\n\n", keys( %{$msgids} ) ) . "\n\n" );
-
-    $self->merge_msgs( { out_doc => $out_doc, msgids => $msgids } );
+    $self->merge_msgs(
+        {   out_file => $out_file,
+            ent_file => $ent_file,
+            out_doc  => $out_doc,
+            msgids   => $msgids
+        }
+    );
 
     $out_doc->pos( $out_doc->root() );
     foreach my $node ( $out_doc->look_down( 'processed', 1 ) ) {
@@ -225,8 +238,6 @@ sub po2xml {
     my $type = $out_doc->attr("_tag");
     my $text = $out_doc->as_XML();
 
-##    $text =~ s/&#10;//g;
-##    $text =~ s/&#9;//g;
     $text =~ s/&#38;([a-zA-Z-_0-9]+;)/&$1/g;
     $text =~ s/&#38;/&amp;/g;
     $text =~ s/&#60;/&lt;/g;
@@ -243,8 +254,12 @@ sub po2xml {
     open( $OUTDOC, ">:encoding(UTF-8)", "$out_file" )
         || croak( maketext( "Could not open [_1] for output!", $out_file ) );
     print $OUTDOC Publican::Builder::dtd_string(
-        { tag => $type, dtdver => $dtdver, cleaning => 1 } );
-##debug_msg("is utf8 text: " . utf8::is_utf8($text) . "\n");
+        {   tag      => $type,
+            dtdver   => $dtdver,
+            cleaning => 1,
+            ent_file => $ent_file
+        }
+    );
     print( $OUTDOC $text );
     close($OUTDOC);
 
@@ -270,6 +285,7 @@ sub update_po {
         || croak( maketext("surname is a mandatory argument") );
     my $email = delete( $args->{email} )
         || croak( maketext("email is a mandatory argument") );
+    my $previous = delete( $args->{previous} );
 
     if ( %{$args} ) {
         croak(
@@ -340,21 +356,19 @@ sub update_po {
                     $self->merge_po(
                         { po_file => $po_file, pot_file => $pot_file } );
                 }
-                elsif (
-                    system(
-                        "msgmerge",      "--no-wrap", "--quiet",
-                        "--backup=none", "--update",  $po_file,
-                        $pot_file
-                    ) != 0
-                    )
-                {
-                    croak(
-                        maketext(
-                            "Fatal Error: msgmerge failed to merge updates. POT File: [_1]. Po File: [_2]",
-                            $pot_file,
-                            $po_file
-                        )
-                    );
+                else {
+                    my @cmd
+                        = qw(msgmerge --no-wrap --quiet --backup=none --update);
+                    push( @cmd, '--previous' ) if ($previous);
+                    if ( system( @cmd, $po_file, $pot_file ) != 0 ) {
+                        croak(
+                            maketext(
+                                "Fatal Error: msgmerge failed to merge updates. POT File: [_1]. Po File: [_2]",
+                                $pot_file,
+                                $po_file
+                            )
+                        );
+                    }
                 }
             }
 
@@ -417,8 +431,6 @@ sub update_po {
 
 Merge updated POT files in to existing PO files.
 
-TODO Impliment this...
-
 =cut
 
 sub merge_po {
@@ -437,8 +449,8 @@ sub merge_po {
         );
     }
 
-    my $pot_arry      = Locale::PO->load_file_asarray($pot_file);
-    my $po_hash       = Locale::PO->load_file_ashash($po_file);
+    my $pot_arry = Locale::PO->load_file_asarray( $pot_file, 'UTF-8' );
+    my $po_hash       = Locale::PO->load_file_ashash( $po_file, 'UTF-8' );
     my @po_keys       = keys( %{$po_hash} );
     my %po_start_keys = map { $_ => 1 } @po_keys;
     my @out_arry      = ();
@@ -490,7 +502,7 @@ POT:
         push( @out_arry, $po_hash->{$obsolete} );
     }
 
-    Locale::PO->save_file_fromarray( $po_file, \@out_arry );
+    Locale::PO->save_file_fromarray( $po_file, \@out_arry, 'UTF-8' );
 
     return;
 }
@@ -529,6 +541,7 @@ sub update_po_all {
     my ( $self, $args ) = @_;
 
     my $msgmerge  = delete( $args->{msgmerge} );
+    my $previous  = delete( $args->{previous} );
     my $firstname = delete( $args->{firstname} );
     my $surname   = delete( $args->{surname} );
     my $email     = delete( $args->{email} );
@@ -547,6 +560,7 @@ sub update_po_all {
             email     => $email,
             firstname => $firstname,
             surname   => $surname,
+            previous  => $previous,
         }
     );
     return;
@@ -561,6 +575,8 @@ Get the strings to translate from an XML::TreeBuilder object
 sub get_msgs {
     my ( $self, $args ) = @_;
     my $doc = delete( $args->{doc} ) || croak("doc is a mandatory argument");
+    my $filename = delete( $args->{filename} )
+        || croak("filename is a mandatory argument");
 
     if ( %{$args} ) {
         croak(
@@ -601,6 +617,14 @@ sub get_msgs {
                         )
                     );
                 }
+## To allow External_Links to be translatable we need to translate the whole list as
+## the attributes of then  members need to be translated.
+                elsif ( $inner->tag() eq 'simplelist' ) {
+                    $filename eq 'External_Links.pot';
+                }
+                elsif ( $filename eq 'External_Links.pot' ) {
+                    0;
+                }
                 else {
 ## Other IGNOREBLOCKS tags are completely ignored for translation structure.
                     not defined(
@@ -611,23 +635,25 @@ sub get_msgs {
         )
         )
     {
-## BUGBUG is this required here?
-##        next if ( $child->attr('processed') );
-##        $child->attr( 'processed', 1 );
-
         next if ( $child->is_empty );
 
         $trans_node = XML::Element->new( $child->tag() );
 
      # Have to be inside a translatable tag here, so don't need to check again
-        my @matches = $child->look_down(
-            '_tag',
-            qr/$TRANSTAGS/,
-            sub { not defined( $_[0]->look_up( '_tag', qr/$IGNOREBLOCKS/ ) ) }
-        );
+        my @matches;
+        if ( $filename ne 'External_Links.pot' ) {
+            @matches = $child->look_down(
+                '_tag',
+                qr/$TRANSTAGS/,
+                sub {
+                    not defined(
+                        $_[0]->look_up( '_tag', qr/$IGNOREBLOCKS/ ) );
+                }
+            );
+        }
 
     # No Nesting so push all of this nodes content on to the output trans_tree
-        if ( !$#matches ) {
+        if ( $#matches == -1 ) {
             $trans_node->push_content( $child->content_list() );
         }
         else {
@@ -660,7 +686,9 @@ sub get_msgs {
                         if ( !$trans_node->is_empty );
                     $trans_node = XML::Element->new( $child->tag() );
                     $trans_tree->push_content(
-                        $self->get_msgs( { doc => $nested } )->content_list()
+                        $self->get_msgs(
+                            { doc => $nested, filename => $filename }
+                        )->content_list()
                     );
                 }
                 else {
@@ -691,6 +719,9 @@ sub merge_msgs {
         || croak("out_doc is a mandatory argument");
     my $msgids = delete( $args->{msgids} )
         || croak("msgids is a mandatory argument");
+    my $ent_file = delete( $args->{ent_file} );
+    my $out_file = delete( $args->{out_file} )
+        || croak( maketext("out_file is a mandatory argument") );
 
     if ( %{$args} ) {
         croak(
@@ -722,6 +753,14 @@ sub merge_msgs {
                         )
                     );
                 }
+## To allow External_Links to be translatable we need to translate the whole list as
+## the attributes of then  members need to be translated.
+                elsif ( $inner->tag() eq 'simplelist' ) {
+                    $out_file =~ /External_Links\.xml/;
+                }
+                elsif ( $out_file =~ /External_Links\.xml/ ) {
+                    0;
+                }
                 else {
 ## Other IGNOREBLOCKS tags are completely ignored for translation structure.
                     not defined(
@@ -738,15 +777,24 @@ sub merge_msgs {
 
         next if ( $child->is_empty );
 
-        my @matches = $child->look_down(
-            '_tag',
-            qr/$TRANSTAGS/,
-            sub { not defined( $_[0]->look_up( '_tag', qr/$IGNOREBLOCKS/ ) ) }
-        );
+     # Have to be inside a translatable tag here, so don't need to check again
+        my @matches;
+        if ( $out_file !~ /External_Links\.xml/ ) {
+            @matches = $child->look_down(
+                '_tag',
+                qr/$TRANSTAGS/,
+                sub {
+                    not defined(
+                        $_[0]->look_up( '_tag', qr/$IGNOREBLOCKS/ ) );
+                }
+            );
+        }
 
     # No Nesting so push all of this nodes content on to the output trans_tree
-        if ( !$#matches ) {
-            $self->translate( { node => $child, msgids => $msgids } );
+        if ( $#matches == -1 ) {
+            $self->translate(
+                { ent_file => $ent_file, node => $child, msgids => $msgids }
+            );
         }
         else {
 
@@ -771,13 +819,21 @@ sub merge_msgs {
                 {
                     if ( $trans_node && !$trans_node->is_empty ) {
                         $self->translate(
-                            { node => $trans_node, msgids => $msgids } );
+                            {   ent_file => $ent_file,
+                                node     => $trans_node,
+                                msgids   => $msgids
+                            }
+                        );
                         $child->push_content( $trans_node->content_list() );
                         $trans_node->delete();
                         $trans_node = XML::Element->new( $child->tag() );
                     }
                     $self->merge_msgs(
-                        { out_doc => $nested, msgids => $msgids } );
+                        {   out_file => $out_file,
+                            out_doc  => $nested,
+                            msgids   => $msgids
+                        }
+                    );
                     $child->push_content($nested);
                 }
                 else {
@@ -787,7 +843,11 @@ sub merge_msgs {
 
             if ( $trans_node && !$trans_node->is_empty ) {
                 $self->translate(
-                    { node => $trans_node, msgids => $msgids } );
+                    {   ent_file => $ent_file,
+                        node     => $trans_node,
+                        msgids   => $msgids
+                    }
+                );
                 $child->push_content( $trans_node->content_list() );
                 $trans_node->delete();
             }
@@ -809,6 +869,7 @@ sub translate {
         || croak("node is a mandatory argument");
     my $msgids = delete( $args->{msgids} )
         || croak("msgids is a mandatory argument");
+    my $ent_file = delete( $args->{ent_file} );
 
     if ( %{$args} ) {
         croak(
@@ -821,30 +882,16 @@ sub translate {
     my $msgid = $node->as_XML();
     my $tag   = $node->tag();
 
-##debug_msg("msgid 1: $tag, $msgid\n");
+    my $po = new Locale::PO( -msgid => detag( $msgid, $tag ), -msgstr => '' );
+    $msgid = $po->msgid();
+
     my $attr_text = '';
-
-##debug_msg("is utf8 msgid: ".utf8::is_utf8($msgid)."\n");
-    utf8::encode($msgid);
-
-    $msgid = $self->normalise( $msgid, $tag );
-    $msgid = po_format( $msgid, $tag );
-    if ( $tag =~ /$VERBATIM/ ) {
-        $msgid =~ s/\\\\n/\\n/g;
+    my %attrs     = $node->all_attr();
+    foreach my $key ( keys(%attrs) ) {
+        next if ( $key =~ /^_/ );
+        $attr_text .= qq{ $key="$attrs{$key}"};
     }
 
-    #debug_msg("msgid 3: |$msgid| |$tag|\n");
-
-    # If a tag has attributes we need to remove them for comparison as
-    # the PO format does not allow this to be stored
-## BUGBUG document this better
-    if ( $msgid =~ m/^"<$tag([\t ]+[^>]+)>[\t ]*(.*)$/ ) {
-        $attr_text = $1;
-        $msgid     = qq{"$2};
-        $attr_text =~ s/\\//g;
-    }
-
-##debug_msg("msgid 4: $tag, $msgid\n");
     # mixed mode tags, para, caption, can be empty at this point
     if ( $msgid and ( $msgid eq '""' ) ) {
 
@@ -857,19 +904,14 @@ sub translate {
             my $msgstr = $msgids->{$msgid}{msgstr};
             debug_msg("DANGER: found obsolete msg: $msgid\n")
                 if ( $msgstr =~ /^#~/ );
-##debug_msg("is utf8 msgstr 1: " . utf8::is_utf8($msgstr) . "\n");
-            utf8::decode($msgstr);
-##debug_msg("is utf8 msgstr 2: " . utf8::is_utf8($msgstr) . "\n");
-##debug_msg("msgstr: |$msgstr|\n");
 
-            my $repl = po_unformat( $msgstr, $tag );
+            my $repl = $po->dequote($msgstr);
 
-##debug_msg("is utf8 repl: ".utf8::is_utf8($repl)."\n");
-##debug_msg("repl: |$repl|\n");
             my $dtd = Publican::Builder::dtd_string(
                 {   tag      => $tag,
                     dtdver   => $self->{publican}->param('dtdver'),
-                    cleaning => 1
+                    cleaning => 1,
+                    ent_file => $ent_file
                 }
             );
             my $new_tree = Publican::Builder::new_tree();
@@ -878,7 +920,6 @@ sub translate {
             $node->push_content( $new_tree->content_list() );
         }
         else {
-##        debug_msg("WARNING: Un-translated message: '$msgid'\n");
             if ( $msgids->{$msgid}->fuzzy() )
             {    # BUGBUG TEST this is still set
                 logger( maketext("WARNING: Fuzzy message in PO file."), RED );
@@ -896,7 +937,6 @@ sub translate {
         }
     }
     else {
-##         debug_msg("WARNING: Missing message: '\n$msgid\n'\n");
         logger(
             maketext(
                 "WARNING: Message missing from PO file, consider updating your POT and PO files."
@@ -939,42 +979,22 @@ sub print_msgs {
         )
         );
 
-    print( $fh $self->header() );
-
     my %msgs = ();
+    my $po = new Locale::PO( -msgid => '', -msgstr => $self->header() );
+    print( $fh $po->dump() );
+    $msgs{''} = $po;
 
     foreach my $child ( $msg_list->content_list() ) {
-        my $msg_id = $child->as_XML();
-##debug_msg("is utf8 msg_id 1: '" . utf8::is_utf8($msg_id) . "'\n");
-##debug_msg("msg_id: $msg_id\n");
+        my $msg_id = detag( $child->as_XML(), $child->tag() );
 
-##        my $msg_id = $child->as_text();
-        $msg_id = po_format( $self->normalise( $msg_id, $child->tag() ),
-            $child->tag() );
-        next unless $msg_id;
-
-##        $msg_id = qq|"$msg_id"|;
-        next
-            if ( $msg_id eq '' || $msg_id eq '""' || defined $msgs{$msg_id} );
-##debug_msg("is utf8 msg_id 2: '" . utf8::is_utf8($msg_id) . "'\n");
-##debug_msg("msg_id: $msg_id\n\n");
-        $msgs{$msg_id} = 1;
-        if ( $child->tag() =~ /$VERBATIM/ ) {
-            $msg_id =~ s/\\\\n/\\n"\n"/g;
+        # This can be empty if a mixed mode tag only contains a block
+        next if ( $msg_id eq '' );
+        if ( !defined( $msgs{$msg_id} ) ) {
+            my $po = new Locale::PO( -msgid => $msg_id, -msgstr => '' );
+            print( $fh $po->dump() );
+            $msgs{$msg_id} = $po;
         }
-##debug_msg("is utf8 msg_id 3: '" . utf8::is_utf8($msg_id) . "'\n");
-##debug_msg("msg_id: $msg_id\n\n");
-        my $tag = $child->tag();
-        print $fh <<MSGID
-#. Tag: $tag
-#, no-c-format
-msgid $msg_id
-msgstr ""
-
-MSGID
-
     }
-
     close($fh);
 
     return;
@@ -992,128 +1012,71 @@ sub header {
     my $date
         = DateTime->now( time_zone => "local" )->strftime("%Y-%m-%d %H:%M%z");
 
-    my $string = <<POT;
-# 
-# AUTHOR <EMAIL\@ADDRESS>, YEAR.
-#
-msgid ""
-msgstr ""
-"Project-Id-Version: 0\\n"
-"POT-Creation-Date: $date\\n"
-"PO-Revision-Date: $date\\n"
-"Last-Translator: Automatically generated\\n"
-"Language-Team: None\\n"
-"MIME-Version: 1.0\\n"
-"Content-Type: application/x-publican; charset=UTF-8\\n"
-"Content-Transfer-Encoding: 8bit\\n"
+    my $lang = $self->{publican}->param('xml_lang');
+    $lang =~ s/_/-/g;
+    my $pver = $self->{publican}->VERSION;
 
+    my $string = <<POT;
+Project-Id-Version: 0
+POT-Creation-Date: $date
+PO-Revision-Date: $date
+Last-Translator: Automatically generated
+Language-Team: None
+Language: $lang 
+MIME-Version: 1.0
+Content-Type: application/x-publican; charset=UTF-8
+Content-Transfer-Encoding: 8bit
+X-Generator: Publican $pver
 POT
 
     return ($string);
 }
 
-=head2 normalise
-
-Remove extraneous white space.
-
-=cut
-
-sub normalise {
-    my $self = shift;
-    my $norm = shift;
-    my $name = shift;
-    if ( $name =~ /$VERBATIM/ ) {
-        $norm =~ s/\n/\\n/g;    # CR
-    }
-    else {
-        $norm =~ s/\n/ /g;        # CR
-        $norm =~ s/^[\t ]*//g;    # space at start of line
-        $norm =~ s/[\t ]*$//g;    # space at end of line
-        $norm =~ s/[\t ]+/ /g;    # colapse spacing
-    }
-
-    # Escaped entities :(
-## TODO dupe code from XmlClean
-##    $norm =~ s/&#10;//g;
-##    $norm =~ s/&#9;//g;
-##    $norm =~ s/&#38;([a-zA-Z-_0-9]+;)/&$1/g;
-    $norm =~ s/&#38;/&amp;/g;
-##    $norm =~ s/&amp;#x200B;/&#x200B;/g;
-##    $norm =~ s/&#x200B; &#x200B;/ /g;
-##    $norm =~ s/&#x200B; / /g;
-    $norm =~ s/&#60;/&lt;/g;
-    $norm =~ s/&#62;/&gt;/g;
-    $norm =~ s/&#34;/"/g;
-    $norm =~ s/&#39;/'/g;
-    $norm =~ s/&quot;/"/g;
-    $norm =~ s/&apos;/'/g;
-
-##debug_msg("is utf8 norm: '" . utf8::is_utf8($norm) . "'\n");
-##debug_msg("norm: $norm\n\n");
-
-    return $norm;
-}
-
-=head2 po_format
+=head2 detag
 
 Format a string for use in a PO file.
 
 =cut
 
-sub po_format {
+sub detag {
     my $string = shift;
     my $name = shift || '';
 
     debug_msg("unknown tag for: $string") if $name eq '';
 
     if ( $name =~ /$VERBATIM/ ) {
-        $string =~ s/^<$name>//s;    # remove start tag to reduce polution
-        $string =~ s/<\/$name>(?:\\n|[\t ])*$//s
-            ;                        # remove close tag to reduce polution
-        $string =~ s/\\/\\\\/g;  # \ seen as control sequence by msg* programs
-        $string =~ s/\"/\\"/g;   # " seen as special char by msg* programs
+
+        # remove start tag to reduce polution
+        $string =~ s/^<$name[^>]*>//;
+
+        # remove close tag to reduce polution
+        $string =~ s/<\/$name>\s*$//;
+        chomp($string);
     }
     else {
-        $string =~ s/^<$name>[\t ]*//s;  # remove start tag to reduce polution
-        $string
-            =~ s/[\t ]*<\/$name>$//s;    # remove close tag to reduce polution
-        $string =~ s/\\/\\\\/g;  # \ seen as control sequence by msg* programs
-        $string =~ s/\"/\\"/g;   # " seen as special char by msg* programs
+        chomp($string);
+
+        # remove start tag & leading space
+        $string =~ s/^<$name[^>]*>\s*//;
+
+        # remove close tag & trailing
+        $string =~ s/\s*<\/$name>\s*$//;
+
+        $string =~ s/\n/ /g;     # CR
+        $string =~ s/^\s*//g;    # space at start of line
+        $string =~ s/\s*$//g;    # space at end of line
+        $string =~ s/\s+/ /g;    # collapse spacing
     }
-    $string = qq|"$string"|;
 
-    return $string;
-}
+    $string =~ s/&#38;/&amp;/g;
+    $string =~ s/&#60;/&lt;/g;
+    $string =~ s/&#62;/&gt;/g;
+    $string =~ s/&#34;/"/g;
+    $string =~ s/&#39;/'/g;
+    $string =~ s/&quot;/"/g;
+    $string =~ s/&apos;/'/g;
 
-=head2 po_unformat
-
-Remove PO formatting from a string.
-
-=cut
-
-sub po_unformat {
-    my $string = shift;
-    my $name = shift || '';
-
-    if ( $name =~ /$VERBATIM/ ) {
-        $string =~ s/^\"//msg;    # strip sol quotes added by msguniq etc
-        $string =~ s/\"$//msg;    # strip sol quotes added by msguniq etc
-##debug_msg("string: $string");
-        $string =~ s/\\n/\n/msg;  # strip eol quotes added by msguniq etc
-##debug_msg("string: $string");
-        $string =~ s/\\"/\"/msg;  # unescape quotes added by po_format
-        $string =~ s/\\\\/\\/g;   # unescape backslash added by po_format
-    }
-    else {
-        $string =~ s/^\"//msg;      # strip sol quotes added by msguniq etc
-        $string =~ s/\"$//msg;      # strip sol quotes added by msguniq etc
-        $string =~ s/\n//msg;       # strip eol quotes added by msguniq etc
-        $string =~ s/^[\t ]*//msg
-            ;    # strip the leading spaces left from the msgid "" line
-        $string =~ s/\\"/\"/msg;    # unescape quotes added by po_format
-        $string =~ s/\\\\/\\/g;     # unescape backslash added by po_format
-    }
-    return $string;
+    return ($string);
 }
 
 =head2 po_report
@@ -1186,8 +1149,7 @@ sub po_report {
                 $po_stats{msg_count}--;
                 next;
             }
-## More accurate word counts
-##            my $count = () = $msgref->msgid() =~ /\w+/g;
+
             my $count = ()
                 = $msgref->msgid()
                 =~ /(?:\s+|<\/[a-zA-Z]+><[a-zA-Z]+>\S|-|.$)/g;
@@ -1272,14 +1234,17 @@ Publican requires no configuration files or environment variables.
 Carp
 version
 Publican
-File::Copy::Recursive
 File::Path
+Term::ANSIColor
+DateTime
+Locale::PO
+XML::TreeBuilder
+String::Similarity
 
 =head1 INCOMPATIBILITIES
 
 None reported.
 
-
 =head1 BUGS AND LIMITATIONS
 
 No bugs have been reported.
