@@ -19,8 +19,7 @@ use Time::localtime;
 use XML::Simple;
 use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
 use Publican::ConfigData;
-
-our $VERSION = '1.4';
+use HTML::TreeBuilder;
 
 my $DB_NAME           = 'books';
 my $DEFAULT_LANG      = 'en-US';
@@ -82,6 +81,93 @@ my %LANG_NAME = (
     'zh-TW'      => '繁體中文',
 );
 
+my %PARAMS = (
+    toc_path => {
+        descr => maketext(
+            "The path to the directory in which to create the top-level index.html file."
+        ),
+        mandatory => 1
+    },
+    tmpl_path => {
+        descr   => maketext("Full path to the template directory."),
+        default => $DEFAULT_TMPL_PATH
+    },
+    def_lang => {
+        descr => maketext(
+            "The default language for this website. Tables of contents for languages other than the default language will link to documents in the default language when translations are not available."
+        ),
+        default => $DEFAULT_LANG
+    },
+    db_file => {
+        descr => maketext(
+            "The name of the SQLite database file for your site, with the filename extension .db"
+        ),
+        mandatory => 1
+    },
+    host => {
+        descr =>
+            maketext("The web host, may be a full URI or a relative path."),
+        default => "/docs"
+    },
+    search => {
+        descr   => maketext("The HTML to inject in as the site serach."),
+        default => maketext("Google site search")
+    },
+    title => {
+        descr   => maketext("Title used for all site navigation pages."),
+        default => maketext("Documentation")
+    },
+    dump =>
+        { descr => maketext("Dump the publican database to an XML file.") },
+    dump_file => {
+        descr => maketext(
+            "The name of the file to dump the publican database to."),
+        default => $DEFAULT_DUMP_FILE
+    },
+    zip_dump => {
+        descr   => maketext("Zip up the dump file after dumping it"),
+        default => 0
+    },
+    toc_type => {
+        descr => maketext(
+            "Template to use for generagting the web style 1 toc file."),
+        default => "toc",
+        alert   => maketext(
+            'This field is deprecated and will be removed from Publican in the future.'
+        ),
+    },
+    toc_js => {
+        descr =>
+            maketext("The source file to use for JavaScript functionality."),
+        default => "default.js"
+    },
+    manual_toc_update => {
+        descr => maketext(
+            "Stop publican from automatically rebuilding teh web site everytime a book is installed, updated or removed."
+        ),
+        default => "0"
+    },
+    debug => {
+        descr   => maketext("Output extra messages when running publican."),
+        default => "0"
+    },
+    footer => {
+        descr => maketext(
+            "HTML to inject in to the footer of every page on the website."),
+        default => ""
+    },
+    web_style => {
+        descr => maketext(
+            "Publican supports mutliple base styles for websites, this picks one."
+        ),
+        constraint => '[1-2]',
+        default    => "1",
+        alert      => maketext(
+            'This field is deprecated and will be removed from Publican in the future.'
+        )
+    },
+);
+
 # This is required to ensure that the correct localised strings are found when running
 # the commands on an non en-US command line
 my $locale = Publican::Localise->get_handle('en-US')
@@ -127,7 +213,16 @@ my %tmpl_strings = (
     Document_Language     => $locale->maketext('Document Language'),
     Document_Home         => $locale->maketext('Document Home'),
     Product_Documentation => $locale->maketext('Product Documentation'),
-    Support => $locale->maketext('Support'),
+    Support               => $locale->maketext('Support'),
+    SrcIsNewer            => $locale->maketext('English is newer'),
+    SelectYourVersion     => $locale->maketext('Select a version'),
+    SelectYourCategory    => $locale->maketext('Select a category'),
+    ScrollToCategory      => $locale->maketext('Scroll to category'),
+    html_label            => $locale->maketext('HTML (Recommended)'),
+    'html-single_label'   => $locale->maketext('Single-page HTML'),
+    pdf_label             => $locale->maketext('PDF'),
+    epub_label            => $locale->maketext('EPUB'),
+    All                   => $locale->maketext('All'),
 );
 
 sub new {
@@ -153,15 +248,17 @@ sub new {
         )
         );
     my $tmpl_path = $config->param('tmpl_path') || $DEFAULT_TMPL_PATH;
-    my $def_lang  = $config->param('def_lang')  || $DEFAULT_LANG;
-    my $db_file   = $config->param('db_file')   || croak(
+    $tmpl_path .= ":$DEFAULT_TMPL_PATH" if ( $config->param('tmpl_path') );
+
+    my $def_lang = $config->param('def_lang') || $DEFAULT_LANG;
+    my $db_file  = $config->param('db_file')  || croak(
         maketext(
             "[_1] is a mandatory field in a site configuration file. Check [_2] for validity.",
             'db_file',
             $site_config
         )
     );
-    my $host      = $config->param('host')      || 'http://localhost/';
+    my $host      = $config->param('host')      || '/docs';
     my $search    = $config->param('search')    || undef;
     my $title     = $config->param('title')     || 'Documentation';
     my $dump      = $config->param('dump')      || undef;
@@ -197,9 +294,8 @@ sub new {
     $self->{debug}             = $debug;
     $self->{web_style}         = $web_style;
 
-    my $conf = { INCLUDE_PATH => $tmpl_path, };
+    my $conf = { INCLUDE_PATH => $tmpl_path, ENCODING => 'utf8', };
 
-    #    $conf->{DEBUG} = Template::Constants::DEBUG_ALL if ($debug);
     $conf->{DEBUG} = Template::Constants::DEBUG_VARS if ($debug);
 
     # create Template object
@@ -262,9 +358,11 @@ CREATE TABLE IF NOT EXISTS $DB_NAME (
 	version_label text(255),
 	name_label text(255),
 	sort_order INTEGER DEFAULT 50,
-	update_date text(10),
 	subtitle text,
 	abstract text,
+	book_version text(10),
+	book_src_lang text(10),
+	update_date DATETIME default current_timestamp,
 	UNIQUE(language, product, version, name)
 )
 CREATE_TABLE
@@ -303,6 +401,8 @@ sub update_or_add_entry {
     my $subtitle      = delete $arg->{subtitle};
     my $abstract      = delete $arg->{abstract};
     my $sort_order    = delete $arg->{sort_order};
+    my $book_version  = delete $arg->{book_version};
+    my $book_src_lang = delete $arg->{book_src_lang};
 
     if ( %{$arg} ) {
         croak "unknown args: " . join( ", ", keys %{$arg} );
@@ -332,6 +432,8 @@ sub update_or_add_entry {
                 subtitle      => $subtitle,
                 abstract      => $abstract,
                 sort_order    => $sort_order,
+                book_version  => $book_version,
+                book_src_lang => $book_src_lang,
             }
         );
     }
@@ -348,6 +450,8 @@ sub update_or_add_entry {
                 subtitle      => $subtitle,
                 abstract      => $abstract,
                 sort_order    => $sort_order,
+                book_version  => $book_version,
+                book_src_lang => $book_src_lang,
             }
         );
     }
@@ -372,27 +476,30 @@ sub add_entry {
     my $subtitle      = delete $arg->{subtitle};
     my $abstract      = delete $arg->{abstract};
     my $sort_order    = delete $arg->{sort_order};
+    my $book_version  = delete $arg->{book_version};
+    my $book_src_lang = delete $arg->{book_src_lang};
 
     if ( %{$arg} ) {
         croak "unknown args: " . join( ", ", keys %{$arg} );
     }
 
-    my $update_date = DateTime->today()->ymd();
-
     $formats = lc $formats;
     $abstract =~ s/\n/ /gm;
 
     my $sql = <<INSERT_ENTRY;
-        INSERT INTO $DB_NAME 
-               (language, product, version, name, formats, product_label, version_label, name_label, update_date, subtitle, abstract, sort_order) 
-               VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
+        INSERT INTO $DB_NAME ( 
+		language, product, version, name, formats, product_label,
+		version_label, name_label, subtitle, abstract, sort_order,
+		book_version, book_src_lang
+	) 
+	VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
 INSERT_ENTRY
 
     return $self->_dbh->do(
-        $sql,           undef,       $language,    $product,
-        $version,       $name,       $formats,     $product_label,
-        $version_label, $name_label, $update_date, $subtitle,
-        $abstract,      $sort_order
+        $sql,           undef,         $language, $product,
+        $version,       $name,         $formats,  $product_label,
+        $version_label, $name_label,   $subtitle, $abstract,
+        $sort_order,    $book_version, $book_src_lang
     );
 }
 
@@ -414,27 +521,30 @@ sub update_entry {
     my $subtitle      = delete $arg->{subtitle};
     my $abstract      = delete $arg->{abstract};
     my $sort_order    = delete $arg->{sort_order};
+    my $book_version  = delete $arg->{book_version};
+    my $book_src_lang = delete $arg->{book_src_lang};
 
     if ( %{$arg} ) {
         croak "unknown args: " . join( ", ", keys %{$arg} );
     }
 
-    my $update_date = DateTime->today()->ymd();
-
     $formats = lc $formats;
     $abstract =~ s/\n/ /gm;
 
     my $sql = <<INSERT_ENTRY;
         UPDATE $DB_NAME SET
-               language = ?, product = ?, version = ?, name = ?, formats = ?, product_label = ?, version_label = ?, name_label = ?, update_date = ?, subtitle = ?, abstract = ?, sort_order = ?
+		language = ?, product = ?, version = ?, name = ?, formats = ?,
+		product_label = ?, version_label = ?, name_label = ?, update_date = current_timestamp,
+		subtitle = ?, abstract = ?, sort_order = ?, book_version = ?,
+		book_src_lang = ?
                WHERE ID = $ID
 INSERT_ENTRY
 
     return $self->_dbh->do(
-        $sql,           undef,       $language,    $product,
-        $version,       $name,       $formats,     $product_label,
-        $version_label, $name_label, $update_date, $subtitle,
-        $abstract,      $sort_order
+        $sql,           undef,         $language, $product,
+        $version,       $name,         $formats,  $product_label,
+        $version_label, $name_label,   $subtitle, $abstract,
+        $sort_order,    $book_version, $book_src_lang
     );
 }
 
@@ -514,26 +624,33 @@ sub get_hash_ref {
     $direction = 'ASC' if ( $def_lang gt $language );
 
     my $sql = <<GET_LIST;
-        SELECT ID,
-               language, 
-               product, 
-               version, 
-               name, 
-               formats,
-               product_label, 
-               version_label, 
-               name_label,
-               update_date,
-               subtitle,
-               abstract,
-               sort_order
-          FROM $DB_NAME 
-         WHERE (language = ? or language = ?)
-      GROUP BY language, product, version, name
-      ORDER BY language $direction,
-               product, 
-               version DESC, 
-               name
+        SELECT $DB_NAME.ID,
+               $DB_NAME.language, 
+               $DB_NAME.product, 
+               $DB_NAME.version, 
+               $DB_NAME.name, 
+               $DB_NAME.formats,
+               $DB_NAME.product_label, 
+               $DB_NAME.version_label, 
+               $DB_NAME.name_label,
+               $DB_NAME.update_date,
+               $DB_NAME.subtitle,
+               $DB_NAME.abstract,
+               $DB_NAME.sort_order,
+               $DB_NAME.book_version,
+               b2.book_version as orig_ver
+          FROM $DB_NAME
+          LEFT JOIN $DB_NAME as b2 ON
+                    $DB_NAME.book_src_lang = b2.language
+		AND $DB_NAME.product = b2.product
+		AND $DB_NAME.version = b2.version
+		AND $DB_NAME.name =b2.name
+         WHERE ($DB_NAME.language = ? or $DB_NAME.language = ?)
+      GROUP BY $DB_NAME.language, $DB_NAME.product, $DB_NAME.version, $DB_NAME.name
+      ORDER BY $DB_NAME.language $direction,
+               $DB_NAME.product, 
+               $DB_NAME.version DESC, 
+               $DB_NAME.name
 GET_LIST
 
     my $sth = $self->_dbh->prepare($sql);
@@ -542,10 +659,10 @@ GET_LIST
     my %list;
 
     while (
-        my ($id,         $lang,        $product,       $version,
-            $name,       $formats,     $product_label, $version_label,
-            $name_label, $update_date, $subtitle,      $abstract,
-            $sort_order
+        my ($id,         $lang,         $product,       $version,
+            $name,       $formats,      $product_label, $version_label,
+            $name_label, $update_date,  $subtitle,      $abstract,
+            $sort_order, $book_version, $orig_ver
         )
         = $sth->fetchrow()
         )
@@ -571,7 +688,11 @@ GET_LIST
             || '2000-01-01';
         $list{$product}{$version}{$name}{subtitle}   = $subtitle;
         $list{$product}{$version}{$name}{abstract}   = $abstract;
-        $list{$product}{$version}{$name}{sort_order} = $sort_order;
+        $list{$product}{$version}{$name}{sort_order} = ( $sort_order || 50 );
+        $list{$product}{$version}{$name}{srcnewer}
+            = !( ( defined($orig_ver) )
+            && ( $orig_ver ne '' )
+            && ( version_sort( $a = $orig_ver, $b = $book_version ) > 0 ) );
     }
 
     $sth->finish();
@@ -638,7 +759,7 @@ sub regen_all_toc {
                 delete( $version->{untrans_books} );
                 push( @versions, $version )
                     if defined( $version->{books} )
-                        && scalar @{ $version->{books} };
+                    && scalar @{ $version->{books} };
             }
 
             if ( scalar @versions ) {
@@ -925,7 +1046,6 @@ SEARCH
                 push( @{$urls}, $url );
             }
 
-#            foreach my $book ( sort( insensitive_sort keys( %{ $list2->{$product}{$version} } ) ) )
             foreach my $book ( i_sort( $list2->{$product}{$version} ) ) {
                 my $book_label = $book;
                 my %book_data;
@@ -942,8 +1062,8 @@ SEARCH
                     $book_label
                         = $list2->{$product}{$version}{$book}{name_label}
                         if ($list2->{$product}{$version}{$book}{name_label}
-                        and $list2->{$product}{$version}{$book}{name_label} ne
-                        $book );
+                        and $list2->{$product}{$version}{$book}{name_label}
+                        ne $book );
 
                     $book_label = decode_utf8($book_label)
                         unless ( is_utf8($book_label) );
@@ -952,8 +1072,9 @@ SEARCH
                         = $list2->{$product}{$version}{$book}{version_label}
                         if (
                             $list2->{$product}{$version}{$book}{version_label}
-                        and $list2->{$product}{$version}{$book}{version_label}
-                        ne $version );
+                        and
+                        $list2->{$product}{$version}{$book}{version_label} ne
+                        $version );
 
                     $version_label = decode_utf8($version_label)
                         unless ( is_utf8($version_label) );
@@ -962,8 +1083,9 @@ SEARCH
                         = $list2->{$product}{$version}{$book}{product_label}
                         if (
                             $list2->{$product}{$version}{$book}{product_label}
-                        and $list2->{$product}{$version}{$book}{product_label}
-                        ne $product );
+                        and
+                        $list2->{$product}{$version}{$book}{product_label} ne
+                        $product );
 
                     $product_label = decode_utf8($product_label)
                         unless ( is_utf8($product_label) );
@@ -1052,7 +1174,7 @@ SEARCH
                     else {
                         print( STDERR
                                 "ERROR: bogus entry found in DB: $lang/$product/$version/$type/$book\n"
-                        );
+                        ) if ( $self->{debug} );
                     }
 
                     push( @types, \%type_data );
@@ -1063,6 +1185,19 @@ SEARCH
                 $book_data{book_clean} =~ s/_/ /g;
                 $book_data{types} = \@types;
 
+                foreach my $format (
+                    sort( insensitive_sort keys(
+                            %{  $list2->{$product}{$version}{$book}{formats}
+                            }
+                    ) )
+                    )
+                {
+                    $book_data{base_format} = $format;
+                    if ( $format =~ m/^html/ ) {
+                        last;
+                    }
+                }
+
                 if ( $lang eq $language ) {
                     push( @books, \%book_data );
                 }
@@ -1146,6 +1281,9 @@ SEARCH
     ) or croak( $self->{Template}->error() );
 
     if ( $self->{web_style} == 1 ) {
+        $vars->{splash}
+            = $self->get_splash(
+            { path => $self->{toc_path} . "/$language" } );
         $vars->{host}  = $host;
         $vars->{lang}  = $language;
         $vars->{title} = $self->{title};
@@ -1159,6 +1297,31 @@ SEARCH
     return \@products;
 }
 
+sub get_splash {
+    my ( $self, $arg ) = @_;
+
+    my $path = delete $arg->{path}
+        || croak "get_splash: file required";
+
+    if ( $arg && %{$arg} ) {
+        croak "unknown args: " . join( ", ", keys %{$arg} );
+    }
+
+    my $html;
+    my $file = $path . "/splash.html";
+    if ( -f ($file) ) {
+        my $tree = HTML::TreeBuilder->new();
+        my $fh;
+        open( $fh, "<:encoding(UTF-8)", $file )
+            || croak(
+            maketext( "Can't open file for html input: [_1]", $! ) );
+        $tree->parse_file($fh);
+        my $node = $tree->look_down( 'class', qr/article/ );
+        $html = $node->as_HTML() if ($node);
+    }
+    return ($html);
+}
+
 sub report {
     my ( $self, $arg ) = @_;
 
@@ -1306,9 +1469,12 @@ sub splash_pages {
         $direction = 'ASC' if ( $def_lang lt $language );
 
         my $sql = <<SQL;
-SELECT distinct product, version, name, language, name_label, version_label, product_label, formats, langs, abstract, subtitle, sort_order
+SELECT distinct product, version, name, language, name_label, version_label, product_label, formats, langs, abstract, subtitle, sort_order, book_version, orig_ver
 FROM (
-  SELECT product, version, name, language, name_label, version_label, product_label, abstract, subtitle, formats, sort_order, 
+  SELECT books.product, books.version, books.name, books.language,
+  books.name_label, books.version_label, books.product_label, books.abstract,
+  books.subtitle, books.formats, books.sort_order, books.book_version,
+  b2.book_version as orig_ver, 
     (
       SELECT GROUP_CONCAT(distinct language)
       FROM books as books2
@@ -1322,9 +1488,14 @@ FROM (
       ORDER BY books2.language
     ) as langs
   FROM books
-  WHERE language = ? OR language = ?
-  GROUP BY product, version, name, language
-  ORDER BY product, version, name, language $direction
+      LEFT JOIN books as b2 ON
+            books.book_src_lang = b2.language
+	AND books.product = b2.product
+	AND books.version = b2.version
+	AND books.name =b2.name
+  WHERE books.language = ? OR books.language = ?
+  GROUP BY books.product, books.version, books.name, books.language
+  ORDER BY books.product, books.version, books.name, books.language $direction
 )
 GROUP BY product, version, name
 ORDER BY product, version desc, name
@@ -1383,9 +1554,7 @@ SQL
             {
                 $record->{$key} = decode_utf8( $record->{$key} )
                     unless ( is_utf8( $record->{$key} ) );
-##                print( STDERR "$key: " . ( $record->{$key} || "" ) . ", " );
             }
-##            print( STDERR "\n\n" );
 
             if ( $product ne '' and ( $product ne $record->{product} ) ) {
 
@@ -1475,7 +1644,6 @@ SQL
 
             my @lang_array;
             if ( $record->{langs} ) {
-##print( STDERR "langs: " . $record->{langs} ."\n\n");
                 foreach my $trans ( sort( split( /,/, $record->{langs} ) ) ) {
                     my %lang_hash = (
                         lang      => $trans,
@@ -1504,7 +1672,16 @@ SQL
             $book_lang_vars->{trans_strings} = $vars;
             $book_lang_vars->{subtitle}      = $record->{subtitle};
             $book_lang_vars->{book_label} =~ s/_/ /g;
-            $book_lang_vars->{sort_order} = $record->{sort_order};
+            $book_lang_vars->{sort_order} = ( $record->{sort_order} || 50 );
+
+            foreach my $format (
+                sort ( insensitive_sort split( /,/, $record->{formats} ) ) )
+            {
+                $book_lang_vars->{base_format} = $format;
+                if ( $format =~ m/^html/ ) {
+                    last;
+                }
+            }
 
             if ( defined $record->{name_label}
                 && $record->{name_label} ne "" )
@@ -1518,6 +1695,17 @@ SQL
             $book_lang_vars->{book_clean} =~ s/^\s*//g;
             $book_lang_vars->{langs} = \@lang_array;
 
+            if (   ( defined( $record->{orig_ver} ) )
+                && ( $record->{orig_ver} ne '' ) )
+            {
+                my $val = version_sort(
+                    $a = $record->{orig_ver},
+                    $b = $record->{book_version}
+                );
+
+                $book_lang_vars->{srcnewer} = ( $val > 0 );
+            }
+
             $self->{Template}->process(
                 'books_lang_menu.tmpl',
                 $book_lang_vars,
@@ -1529,11 +1717,27 @@ SQL
             $book_list{$product}{$version}{ $record->{name} }
                 = $book_lang_vars;
 
+            my %formats;
+            map( $formats{$_} = 1, split( /,/, $record->{formats} ) );
+
+            my @forder = qw(html html-single epub pdf);
+            foreach my $format (@forder) {
+                if ( defined( $formats{$format} ) ) {
+                    push(
+                        @{  $book_ver_list{$product}{ $record->{name} }
+                                {$version}{formats}
+                        },
+                        $format
+                    );
+                    delete( $formats{$format} );
+                }
+            }
+
             push(
                 @{  $book_ver_list{$product}{ $record->{name} }{$version}
                         {formats}
-                    },
-                split( /,/, $record->{formats} )
+                },
+                keys(%formats)
             );
         }
 
@@ -1631,9 +1835,35 @@ sub write_version_index {
         croak "unknown args: " . join( ", ", keys %{$arg} );
     }
 
+    my $file = Publican::ConfigData->config('templates')
+        . "/groups/$lang/$product/External_Links.xml";
+    if ( -f $file ) {
+        my $xml_doc = XML::TreeBuilder->new(
+            { 'NoExpand' => "1", 'ErrorContext' => "2" } );
+        eval { $xml_doc->parse_file($file); };
+        croak( maketext( "Can't open file '[_1]' [_2]", $file, $@ ) ) if ($@);
+        foreach my $node (
+            $xml_doc->look_down( '_tag' => 'member', version => $version ) )
+        {
+            my $book_lang_vars;
+            $book_lang_vars->{product}    = $product;
+            $book_lang_vars->{version}    = $version;
+            $book_lang_vars->{lang}       = $lang;
+            $book_lang_vars->{name}       = $node->attr('title');
+            $book_lang_vars->{subtitle}   = $node->as_trimmed_text();
+            $book_lang_vars->{sort_order} = ( $node->attr('role') || 50 );
+            $book_lang_vars->{external}   = 1;
+            $book_lang_vars->{uri}        = $node->attr('href');
+            $book_list->{ $node->attr('title') } = $book_lang_vars;
+        }
+    }
+
     my $host = $self->{host};
 
     my $index_vars;
+    $index_vars->{style}            = $self->{web_style};
+    $index_vars->{v_sort}           = \&v_sort;
+    $index_vars->{i_sort}           = \&i_sort;
     $index_vars->{product}          = $product;
     $index_vars->{version}          = $version;
     $index_vars->{host}             = $host;
@@ -1647,6 +1877,8 @@ sub write_version_index {
     $index_vars->{footer}           = $self->{footer};
     $index_vars->{site_title}       = $self->{title};
     $index_vars->{book_ver_list}    = $book_ver_list;
+    $index_vars->{splash}           = $self->get_splash(
+        { path => $self->{toc_path} . "/$lang/$product/$version" } );
 
     $self->{Template}->process(
         'versions_index.tmpl', $index_vars,
@@ -1691,6 +1923,7 @@ sub write_product_index {
     my $host = $self->{host};
 
     my $index_vars;
+    $index_vars->{style}         = $self->{web_style};
     $index_vars->{product}       = $product;
     $index_vars->{host}          = $host;
     $index_vars->{book_list}     = $book_list;
@@ -1704,6 +1937,11 @@ sub write_product_index {
     $index_vars->{footer}        = $self->{footer};
     $index_vars->{site_title}    = $self->{title};
     $index_vars->{book_ver_list} = $book_ver_list;
+    $index_vars->{splash}
+        = $self->get_splash(
+        { path => $self->{toc_path} . "/$lang/$product" } );
+    $index_vars->{categories}
+        = ( -d "$DEFAULT_TMPL_PATH/groups/$lang/$product" );
 
     $self->{Template}->process(
         'products_index.tmpl', $index_vars,
@@ -1746,6 +1984,7 @@ sub write_product_menu {
     my @products = sort( insensitive_sort keys( %{$book_list} ) );
 
     my $index_vars;
+    $index_vars->{style}         = $self->{web_style};
     $index_vars->{host}          = $host;
     $index_vars->{book_list}     = $book_list;
     $index_vars->{lang}          = $lang;
@@ -1782,7 +2021,7 @@ sub i_sort {
                         <=> ( $hash->{$b}->{sort_order} || 50 );
                 }
                 else { lc($a) cmp lc($b) }
-            } keys( %{$hash} ) )
+        } keys( %{$hash} ) )
     );
 }
 
@@ -1808,6 +2047,7 @@ sub write_language_index {
     my @products = sort( insensitive_sort keys( %{$book_list} ) );
 
     my $index_vars;
+    $index_vars->{style}         = $self->{web_style};
     $index_vars->{host}          = $host;
     $index_vars->{book_list}     = $book_list;
     $index_vars->{lang}          = $lang;
@@ -1821,6 +2061,8 @@ sub write_language_index {
     $index_vars->{trans_strings} = $trans_strings;
     $index_vars->{footer}        = $self->{footer};
     $index_vars->{site_title}    = $self->{title};
+    $index_vars->{splash}
+        = $self->get_splash( { path => $self->{toc_path} . "/$lang" } );
 
     $self->{Template}->process(
         'language_index.tmpl', $index_vars,
@@ -1849,6 +2091,7 @@ sub write_language_labels {
     my $host = $self->{host};
 
     my $index_vars;
+    $index_vars->{style}         = $self->{web_style};
     $index_vars->{lang}          = $lang;
     $index_vars->{labels}        = $labels;
     $index_vars->{trans_strings} = $trans_strings;
@@ -1897,7 +2140,8 @@ sub write_books_index {
             foreach my $version (@versions) {
 
                 my $index_vars;
-                $index_vars->{host} = $host;
+                $index_vars->{style} = $self->{web_style};
+                $index_vars->{host}  = $host;
                 $index_vars->{book} = $book_list->{$product}{$version}{$book};
                 $index_vars->{lang} = $lang;
                 $index_vars->{langs}    = $langs;
@@ -2021,51 +2265,89 @@ SQL
     return;
 }
 
+sub site_params_as_docbook {
+    my ($web_list) = @_;
+
+    foreach my $key ( sort( keys(%PARAMS) ) ) {
+        my $entry = XML::Element->new_from_lol(
+            [ 'varlistentry', { id => "site_$key" }, [ 'term', "$key" ] ] );
+
+        $web_list->push_content($entry);
+        my $item = XML::Element->new_from_lol(
+            [ 'listitem', [ 'para', $PARAMS{$key}->{descr} ] ] );
+
+        $entry->push_content($item);
+
+        if ( defined( $PARAMS{$key}->{default} ) ) {
+            my $def = XML::Element->new_from_lol(
+                [   'para',
+                    maketext(
+                        "The default value for this parameter is: [_1]",
+                        $PARAMS{$key}->{default}
+                    )
+                ]
+            );
+
+            $item->push_content($def);
+        }
+
+        if ( defined( $PARAMS{$key}->{constraint} ) ) {
+            my $constraint = XML::Element->new_from_lol(
+                [   'para',
+                    maketext(
+                        "This parameter is constrained with the following regular expression: [_1]",
+                        $PARAMS{$key}->{constraint}
+                    )
+                ]
+            );
+
+            $item->push_content($constraint);
+        }
+
+        if ( defined( $PARAMS{$key}->{not_for} ) ) {
+            my $info = XML::Element->new_from_lol(
+                [   'tip',
+                    [   'para',
+                        maketext(
+                            "This field is not supported for: [_1].",
+                            $PARAMS{$key}->{not_for}
+                        )
+                    ]
+                ]
+            );
+
+            $item->push_content($info);
+        }
+
+        if ( defined( $PARAMS{$key}->{alert} ) ) {
+            my $warn = XML::Element->new_from_lol(
+                [ 'warning', [ 'para', $PARAMS{$key}->{alert} ] ] );
+
+            $item->push_content($warn);
+        }
+    }
+
+}
+
 1;    # Magic true value required at end of module
 __END__
 
+=encoding utf8
+
 =head1 NAME
 
 Publican::WebSite - Manage a documentation website
 
-
-=head1 VERSION
-
-This document describes Publican::WebSite version 0.0.1
-
-
 =head1 SYNOPSIS
 
     use Publican::WebSite;
 
-=for author to fill in:
-    Brief code example(s) here showing commonest usage(s).
-    This section will be as far as many users bother reading
-    so make it as educational and exeplary as possible.
-  
-    my $ws = Publican::WebSite::new();
-
-	$ws->add($product, $version, $name, $format);
-
-    foreach my $entry ($ws->list()) { ... }
-
-    $ws->del($product, $version, $name, $format);
-
 =head1 DESCRIPTION
 
-=for author to fill in:
-    Write a full description of the module and its features here.
-    Use subsections (=head2, =head3) as appropriate.
-
+Builds the static navgation content for a Publican Web Site.
 
 =head1 INTERFACE 
 
-=for author to fill in:
-    Write a separate section listing the public components of the modules
-    interface. These normally consist of either subroutines that may be
-    exported, or methods that may be called on objects belonging to the
-    classes provided by the module.
-
 =head2  new
 
 Create a Publican::WebSite object...
@@ -2184,6 +2466,14 @@ Sort strings in a case insensitive order.
 
 Sort version strings in correct order. Handles X Vs X.Y Vs X.Y.Z.
 
+=head2 get_splash
+
+Returns the splash page header as a HTML chunk.
+
+=head2 site_params_as_docbook
+
+Returns the site paramaters as a docbook chunk, used for documeting options.
+
 =head1 AUTHOR
 
 Jeff Fearn  C<< <jfearn@redhat.com> >>
