@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use 5.008;
 use Carp;
-use XML::TreeBuilder;
+use XML::TreeBuilder 5.1;
 use Config::Simple '-strict';
 use Publican;
 use File::Path;
@@ -12,25 +12,13 @@ use Term::ANSIColor qw(:constants);
 use Publican::Builder;
 use File::Inplace;
 use DBI;
-
-## Testing collation
-my $test_collate = 1;
-if ($test_collate) {
-    use Unicode::Collate;
-}
-
-use vars qw( $VERSION );
-
-$VERSION = '0.2';
+use Data::Dumper;
+use File::Basename;
 
 =head1 NAME
 
 Publican::XmlClean - A module to reformat XML to Publican standards
 
-=head1 VERSION
-
-This document describes Publican::XmlClean version $VERSION
-
 =head1 SYNOPSIS
 
     use Publican::XmlClean;
@@ -54,8 +42,8 @@ my %UPDATED_IDS;
 my %MAX_CONFORMANCE;
 
 my %MAP_OUT = (
-    section    => { block => 1, newline_after => 1 },
-    refsection => { block => 1, newline_after => 1, no_id => 1 },
+    section    => { block => 1, },
+    refsection => { block => 1, no_id => 1 },
     chapter       => { block         => 1 },
     preface       => { block         => 1 },
     bibliography  => { block         => 1 },
@@ -116,14 +104,13 @@ my %MAP_OUT = (
     imagedata     => {},
     'xi:include'  => { newline_after => 1 },
     'xi:fallback' => { newline_after => 1 },
-
-    #    glossary          => { block         => 1 },
-    #    glossentry        => { block         => 1, id_node => 'glossterm' },
-    #    glossdiv          => { block         => 1 },
-    #    glossdef          => { block         => 1 },
-    #    glossterm         => { newline_after => 1 },
-    #    glosssee          => { newline_after => 1 },
-    #    glossseealso      => { newline_after => 1 },
+    glossary          => { block         => 1 },
+    glossentry        => { block         => 1, id_node => 'glossterm' },
+    glossdiv          => { block         => 1 },
+    glossdef          => { block         => 1 },
+    glossterm         => { newline_after => 1 },
+    glosssee          => { newline_after => 1 },
+    glossseealso      => { newline_after => 1 },
     table             => { block         => 1 },
     informaltable     => { block         => 1 },
     thead             => { block         => 1 },
@@ -132,7 +119,7 @@ my %MAP_OUT = (
     tr                => { block         => 1 },
     td                => { block         => 1 },
     row               => { block         => 1 },
-    entry             => { block         => 1 },
+    entry             => { newline_after => 1 },
     refentry          => { block         => 1 },
     refmeta           => { block         => 1 },
     refentryinfo      => { block         => 1, no_id => 1 },
@@ -198,20 +185,20 @@ my %MAP_OUT = (
     computeroutput => {},
     guimenuitem    => {},
     textobject     => { block => 1 },
-    varlistentry   => { block => 1 },
-    term           => { newline_after => 1 },
-    colspec        => { newline_after => 1 },
-    areaspec       => { block => 1 },
-    areaset        => { block => 1, keep_id => 1 },
-    area           => { newline_after => 1, keep_id => 1 },
-    calloutlist  => { block => 1 },
-    callout      => { block => 1 },
-    procedure    => { block => 1, newline_after => 1 },
-    appendix     => { block => 1 },
-    appendixinfo => { block => 1 },
-    cmdsynopsis  => { block => 1 },
-    arg          => { block => 1 },
-    group        => { block => 1 },
+    varlistentry   => { block => 1, id_node => 'term' },
+    term         => { newline_after => 1 },
+    colspec      => { newline_after => 1 },
+    areaspec     => { block         => 1 },
+    areaset      => { block         => 1, keep_id => 1 },
+    area         => { newline_after => 1, keep_id => 1 },
+    calloutlist  => { block         => 1 },
+    callout      => { block         => 1 },
+    procedure    => { block         => 1 },
+    appendix     => { block         => 1 },
+    appendixinfo => { block         => 1 },
+    cmdsynopsis  => { block         => 1 },
+    arg          => { block         => 1 },
+    group        => { block         => 1 },
     accel         => {},
     blockquote    => { block => 1 },
     classname     => {},
@@ -252,6 +239,7 @@ my %MAP_OUT = (
     '~comment'    => {},
     foreignphrase => {},
     chapterinfo   => { block => 1 },
+    info          => { block => 1 },
     keywordset    => { block => 1 },
     keyword       => { newline_after => 1 },
     subjectset    => { block => 1 },
@@ -279,12 +267,13 @@ sub new {
         if ( $args->{update_includes} );
     $config->param( 'clean_id', ( delete( $args->{clean_id} ) ) || 0 );
     $config->param( 'donotset_lang',
-        ( delete( $args->{donotset_lang} ) ) || 1 );
+        ( delete( $args->{donotset_lang} ) ) || 0 );
     $config->param( 'distributed_set',
         ( delete( $args->{distributed_set} ) ) || 0 );
     $config->param( 'exclude_ent', ( delete( $args->{exclude_ent} ) ) || 0 );
     $config->param( 'cleaning',
         ( delete( $args->{cleaning} ) ) || $config->param('clean_id') );
+    $config->param( 'id_attr', ( delete( $args->{id_attr} ) ) || 'id' );
 
     if ( %{$args} ) {
         croak(
@@ -387,9 +376,10 @@ sub Clean_ID {
 
         # keep_id means keep the current ID without modification.
         if ( $MAP_OUT{$tag}->{keep_id} ) {
-            $my_id = $node->id() || "";
+            $my_id = $node->attr( $self->{config}->param('id_attr') ) || "";
         }
         elsif ( !$MAP_OUT{$tag}->{no_id} ) {
+## TODO this doesn;t work when teh title is in an info. <info><title>...
             foreach my $child ( $node->content_refs_list() ) {
                 if ( ref $$child
                     && $$child->{'_tag'} eq
@@ -422,20 +412,27 @@ sub Clean_ID {
 
         # prepend product & book name (to avoid problems in sets)
         # prepend tag type for translations BZ #427312
-        if ( $my_id ne "" ) {
+        # If $product is unset then we are dealing with common content, so don't do the above.
+        if ( $my_id ne "" && $product ) {
             $my_id = "$product-$docname-$my_id";
             $my_id = substr( $tag, 0, 4 ) . "-$my_id";
         }
 
-        if ( $node->id() && $node->id() ne $my_id ) {
-            $UPDATED_IDS{ $node->id() } = $my_id;
+        $my_id = $node->attr( $self->{config}->param('id_attr') ) if ( !$my_id  || ($my_id eq "" ));
+
+
+        if (   $node->attr( $self->{config}->param('id_attr') )
+            && $node->attr( $self->{config}->param('id_attr') ) ne $my_id )
+        {
+            $UPDATED_IDS{ $node->attr( $self->{config}->param('id_attr') ) }
+                = $my_id;
         }
 
-        if ( $my_id eq "" ) {
+        if ( !$my_id || ( $my_id eq "") ) {
             $my_id = undef;
         }
 
-        $node->attr( 'id', $my_id );
+        $node->attr( $self->{config}->param('id_attr'), $my_id );
     }
 
     return;
@@ -465,7 +462,6 @@ sub print_xml {
         );
     }
 
-    my $lvl       = 0;
     my $dtdver    = $self->{publican}->param('dtdver');
     my $docname   = $self->{publican}->param('docname');
     my $lang      = $self->{config}->param('lang');
@@ -495,9 +491,42 @@ sub print_xml {
         }
 
         if ( $dtdver =~ m/^5/ ) {
-            $xml_doc->attr( 'xml:lang', $lang );
+            $xml_doc->attr( 'xmlns',       'http://docbook.org/ns/docbook' );
+            $xml_doc->attr( 'xmlns:xlink', 'http://www.w3.org/1999/xlink' );
+            $xml_doc->attr( 'version',     $dtdver );
+            $xml_doc->attr( 'xml:lang',    $lang );
         }
 
+        my $doctype;
+        foreach my $node ( $xml_doc->look_down( '_tag', '~declaration' ) ) {
+            $node->detach();
+            unless ($docname) {
+
+                if ( $node->attr('type') && $node->attr('type') eq 'DOCTYPE' )
+                {
+                    $doctype = q|<!DOCTYPE | . $node->attr('mytag');
+                    $doctype .= q| PUBLIC "| . $node->attr('pid') . q|"|
+                        if ( $node->attr('pid')
+                        && $node->attr('pid') ne 'pid' );
+                    $doctype .= q| "| . $node->attr('uri') . q|"|
+                        if ( $node->attr('uri')
+                        && $node->attr('uri') ne 'uri' );
+                    $doctype .= qq| [\n|;
+                }
+                else {
+                    $doctype
+                        .= q|<!ENTITY |
+                        . $node->attr('name') . q| "|
+                        . $node->attr('value')
+                        . qq|">\n|;
+                }
+            }
+
+##             print(STDERR Dumper($node));
+        }
+
+        $doctype .= qq|]>\n| if ($doctype);
+
         my $type = $xml_doc->attr("_tag");
         $file =~ m|^(.*/xml/)|;
         my $text = $self->my_as_XML(
@@ -522,22 +551,24 @@ sub print_xml {
         if ( $docname && !$self->{config}->param('exclude_ent') ) {
             my $xml_lang = $self->{publican}->param('xml_lang');
             if ( -e "$xml_lang/$main_file.ent" ) {
-                $ent_file = "$xml_lang/$main_file.ent";
-
-                # damn entities need special handling for when people include
-                # entities in abstracts.
-                $ent_file = "$main_file.ent" if ($cleaning);
+                $ent_file = "$main_file.ent";
+                $ent_file = "$path$main_file.ent" if ($path);
             }
         }
 
-        print( $OUTDOC Publican::dtd_string(
-                {   tag      => $type,
-                    dtdver   => $dtdver,
-                    ent_file => $ent_file,
-                    cleaning => $cleaning,
-                }
-            )
-        );
+        if ($doctype) {
+            print( $OUTDOC $doctype );
+        }
+        else {
+            print( $OUTDOC Publican::dtd_string(
+                    {   tag      => $type,
+                        dtdver   => $dtdver,
+                        ent_file => $ent_file,
+                        cleaning => $cleaning,
+                    }
+                )
+            );
+        }
 
         #debug_msg("is utf8: " . utf8::is_utf8($text) . "\n");
         print( $OUTDOC $text );
@@ -606,8 +637,8 @@ sub my_as_XML {
     $tree->traverse(
         sub {
             ( $node, $start ) = @_;
-            if ( ref $node ) {     # it's an element
-                                   # delete internal attrs
+            if ( ref $node ) {    # it's an element
+                                  # delete internal attrs
                 $node->attr( 'depth', undef );
                 $node->attr( 'name',  undef );
 
@@ -615,7 +646,7 @@ sub my_as_XML {
 
                 #print(STDERR "tag: $tag\n");
 
-                if ($start) {      # on the way in
+                if ($start) {     # on the way in
                     if ( $banned_tags{$tag} ) {
                         croak(
                             maketext(
@@ -645,7 +676,8 @@ sub my_as_XML {
                     if (( $MAP_OUT{$tag}->{'newline'} )
                         && (   ( not defined $MAP_OUT{$tag}->{mixed_mode} )
                             || ( not $MAP_OUT{$tag}->{mixed_mode} )
-                            || ( not $node->look_up( '_tag', 'para' ) ) )
+                            || (not $node->look_up( '_tag', qr/para|entry/ ) )
+                        )
                         )
                     {
                         push( @xml, "\n", $indent x $depth );
@@ -659,9 +691,11 @@ sub my_as_XML {
                    # Check to make sure the block is starting on it's own line
                    # If not add a new line and indent
                         if (( $xml[$#xml] && $xml[$#xml] =~ /\S/ )
-                            && ( ( not defined $MAP_OUT{$tag}->{mixed_mode} )
+                            && (( not defined $MAP_OUT{$tag}->{mixed_mode} )
                                 || ( not $MAP_OUT{$tag}->{mixed_mode} )
-                                || ( not $node->look_up( '_tag', 'para' ) ) )
+                                || (not $node->look_up( '_tag',
+                                        qr/para|entry/ ) )
+                            )
                             )
                         {
                             push( @xml, "\n", $indent x $depth );
@@ -718,9 +752,11 @@ sub my_as_XML {
                     {
                         push( @xml, $node->starttag_XML( undef, 1 ) );
                         if ($MAP_OUT{$tag}->{newline_after}
-                            && ( ( not defined $MAP_OUT{$tag}->{mixed_mode} )
+                            && (( not defined $MAP_OUT{$tag}->{mixed_mode} )
                                 || ( not $MAP_OUT{$tag}->{mixed_mode} )
-                                || ( not $node->look_up( '_tag', 'para' ) ) )
+                                || (not $node->look_up( '_tag',
+                                        qr/para|entry/ ) )
+                            )
                             )
                         {
                             push( @xml, "\n", $indent x $depth );
@@ -739,9 +775,11 @@ sub my_as_XML {
                         }
                         elsif (
                             ( not $MAP_OUT{$tag}->{verbatim} )
-                            && ( ( not defined $MAP_OUT{$tag}->{mixed_mode} )
+                            && (( not defined $MAP_OUT{$tag}->{mixed_mode} )
                                 || ( not $MAP_OUT{$tag}->{mixed_mode} )
-                                || ( not $node->look_up( '_tag', 'para' ) ) )
+                                || (not $node->look_up( '_tag',
+                                        qr/para|entry/ ) )
+                            )
                             )
                         {
                             push( @xml, "\n", $indent x $depth );
@@ -772,7 +810,8 @@ sub my_as_XML {
                             }
                             elsif (( defined $MAP_OUT{$tag}->{mixed_mode} )
                                 && ( $MAP_OUT{$tag}->{mixed_mode} )
-                                && ( $node->look_up( '_tag', 'para' ) ) )
+                                && ($node->look_up( '_tag', qr/para|entry/ ) )
+                                )
                             {
                                 $depth--;
                             }
@@ -794,7 +833,7 @@ sub my_as_XML {
                         and !@{ $node->content_array_ref() } )
                     {
                         if ( $tag eq '~cdata' ) {
-                            push( @xml, qq|]]>\n| );
+                            push( @xml, qq|]]>| );
                         }
                         else {
                             push( @xml, $node->endtag_XML() );
@@ -804,7 +843,8 @@ sub my_as_XML {
                     if (( $MAP_OUT{$tag}->{newline_after} )
                         && (   ( not defined $MAP_OUT{$tag}->{mixed_mode} )
                             || ( not $MAP_OUT{$tag}->{mixed_mode} )
-                            || ( not $node->look_up( '_tag', 'para' ) ) )
+                            || (not $node->look_up( '_tag', qr/para|entry/ ) )
+                        )
                         )
                     {
                         push( @xml, "\n", $indent x $depth );
@@ -813,7 +853,8 @@ sub my_as_XML {
                     if (( $MAP_OUT{$tag}->{block} )
                         && (   ( not defined $MAP_OUT{$tag}->{mixed_mode} )
                             || ( not $MAP_OUT{$tag}->{mixed_mode} )
-                            || ( not $node->look_up( '_tag', 'para' ) ) )
+                            || (not $node->look_up( '_tag', qr/para|entry/ ) )
+                        )
                         )
                     {
                         push( @xml, "\n", $indent x $depth );
@@ -913,12 +954,14 @@ sub validate_tables {
                 ) . "\n";
 
             foreach my $row ( $node->look_down( "_tag", "row" ) ) {
-                my @entries = $row->look_down( "_tag", "entry" );
+                my @entries = $row->look_down( "_tag", qr/(?:entry|entrytbl)/,
+                    sub { $_[0]->depth() == ( $row->depth() + 1 ) } );
+## BUGBUG this is incomplete, it should check colwidths
                 if ( @entries > $cols ) {
                     croak maketext(
                         "*ERROR: Fatal Table Error* Table ([_1]) contains invalid data\nAttribute cols ([_2]) does not match number of entry elements ([_3])",
-                        $title, $cols, @entries )
-                        . "\n";
+                        $title, $cols, scalar @entries
+                    ) . "\n";
                 }
             }
         }
@@ -927,40 +970,6 @@ sub validate_tables {
     return;
 }
 
-=head2 sort_glossaries
-
-Sort glosslists
-
-=cut
-
-sub sort_glossaries {
-    my ( $self, $xml_doc ) = @_;
-
-    return unless ($test_collate);
-
-    my $Collator = Unicode::Collate->new();
-
-    if ($xml_doc) {
-        foreach
-            my $glosslist ( $xml_doc->root->look_down( "_tag", "glosslist" ) )
-        {
-            my @glossentries = sort( {
-                    $Collator->cmp(
-                        $a->look_down( "_tag", "glossterm" )->as_text(),
-                        $b->look_down( "_tag", "glossterm" )->as_text()
-                    )
-            } $glosslist->look_down( "_tag", "glossentry" ) );
-
-            foreach my $glossentry (@glossentries) {
-                $glossentry->detach();
-                $glosslist->push_content($glossentry);
-            }
-        }
-    }
-
-    return;
-}
-
 =head2 process_file
 
 Create XML::TreeBuilder object and perform operations.
@@ -996,14 +1005,14 @@ sub process_file {
     $xml_doc->store_comments(1);
     $xml_doc->store_pis(1);
     $xml_doc->store_cdata(1);
+    $xml_doc->store_declarations(1);
 ##debug_msg("here 1");
+    eval { $xml_doc->parse_file($file); };
 
-    $xml_doc->parse_file($file)
-        || croak( maketext( "Can't open file '[_1]' [_2]", $file, $@ ) );
+    croak( maketext( "Can't open file '[_1]' [_2]", $file, $@ ) ) if ($@);
 ##debug_msg("here 2");
 
     $self->validate_tables($xml_doc);
-    $self->sort_glossaries($xml_doc);
 
     if ($update_includes) {
         $self->update_include($xml_doc);
@@ -1114,27 +1123,12 @@ sub set_unique_ids {
     foreach my $xml_file ( sort(@xml_files) ) {
         next if ( $xml_file =~ m{/extras/} );
 
-        # get declarations
-        my $INDOC;
-        open( $INDOC, "<:encoding(UTF-8)", "$xml_file" )
-            || croak(
-            maketext( "Could not open [_1] for output!", $xml_file ) );
-
-        my $declarations = "";
-        my @lines        = <$INDOC>;
-        foreach my $line (@lines) {
-            $declarations .= $line;
-            if ( $line =~ /\]\>\s*$/ ) {
-                last;
-            }
-        }
-
-        $INDOC->close();
-
         my $xml_doc = XML::TreeBuilder->new(
             { 'NoExpand' => "1", 'ErrorContext' => "2" } );
         $xml_doc->store_comments(1);
         $xml_doc->store_pis(1);
+        $xml_doc->store_cdata(1);
+        $xml_doc->store_declarations(1);
 
         $xml_doc->parse_file($xml_file);
 
@@ -1166,7 +1160,7 @@ sub set_unique_ids {
                     $tag = $node->{'_tag'};
 
                     if (   $start
-                        && $node->id()
+                        && $node->attr( $self->{config}->param('id_attr') )
                         && !$node->attr('conformance') )
                     {    # on the way in
                         $node->attr( 'conformance', ++$unique_id );
@@ -1176,21 +1170,7 @@ sub set_unique_ids {
             }
         );
 
-        my $xml = $tree->as_XML();
-        $xml =~ s/&#34;/"/g;
-        $xml =~ s/&#39;/'/g;
-        $xml =~ s/&quot;/"/g;
-        $xml =~ s/&apos;/'/g;
-
-        $xml = $declarations . $xml;
-
-        my $OUTDOC;
-        open( $OUTDOC, ">:encoding(UTF-8)", "$xml_file" )
-            || croak(
-            maketext( "Could not open [_1] for output!", $xml_file ) );
-
-        print( $OUTDOC $xml );
-        close($OUTDOC);
+       $self->print_xml( { xml_doc => $tree, out_file => $xml_file } );
     }
 
     eval {
